@@ -393,7 +393,6 @@ ANTMessage::ANTMessage(ANT *parent, const unsigned char *message) {
             case ANTChannel::CHANNEL_TYPE_POWER:
 
                 channel = message[3];
-                //qDebug()<<channel<<"power event page:"<<data_page;
                 switch (data_page) {
 
                 case ANT_STANDARD_POWER: // 0x10 - standard power
@@ -482,7 +481,7 @@ ANTMessage::ANTMessage(ANT *parent, const unsigned char *message) {
                             autoZeroStatus = message[6] & 0x02;
                             break;
 
-                        default: 
+                        default:
                             break;
 
                     }
@@ -519,6 +518,83 @@ ANTMessage::ANTMessage(ANT *parent, const unsigned char *message) {
                 newsmo2 = 0.1f * double (((message[10] & 0xc0)>>6) + (message[11]<<2));
                 break;
 
+            case ANTChannel::CHANNEL_TYPE_FITNESS_EQUIPMENT:
+                switch (data_page)
+                {
+                case FITNESS_EQUIPMENT_GENERAL_PAGE:
+                     //based on "ANT+ Device Profile Fitness Equipment" rev 4.1 p 42: 6.5.2 Data page 0x10 - General FE Data
+                     fecEqtType = message[5] & 0x1F;
+                     elapsedTime = message[6];                  // elapsed time since begining of workout ; unit 0.25s ; rollover 64s
+                     fecRawDistance = message[7];               // accumulated distance ; unit = m ; rollover = 256m
+                     fecSpeed = (message[9] << 8) | message[8]; // instantaneous speed ; unit=0.001 m/s ; 0xFFFF=invalid
+                     instantHeartrate = message[10];            // instantaneous heartrate ; unit = bpm ; 0xFF=invalid
+                     fecHRSource = message[11] & 0x03;
+                     fecDistanceCapability = (message[11] & 0x04) == 0x04;
+                     fecSpeedIsVirtual     = (message[11] & 0x08) == 0x08;
+                     fecState = (message[11] & 0xF0) >> 4;
+                     break;
+
+                case FITNESS_EQUIPMENT_TRAINER_SPECIFIC_PAGE:
+                     //based on "ANT+ Device Profile Fitness Equipment" rev 4.1 p 58: 6.6.7 Data page 0x19 - Specific trainer data
+                     fecPage0x19EventCount = message[5];
+                     fecCadence = message[6];
+                     fecAccumulatedPower = message[7];
+                     fecAccumulatedPower |= message[8] << 8;
+                     fecInstantPower = message[9];
+                     fecInstantPower |= (message[10] & 0x0F) << 8;
+                     fecPowerCalibRequired = ((message[10] & 0xF0) >> 4) & FITNESS_EQUIPMENT_POWERCALIB_REQU;
+                     fecResisCalibRequired = ((message[10] & 0xF0) >> 4) & FITNESS_EQUIPMENT_RESISCALIB_REQU;
+                     fecUserConfigRequired = ((message[10] & 0xF0) >> 4) & FITNESS_EQUIPMENT_USERCONFIG_REQU;
+                     fecPowerOverLimits = message[11] & 0x0F;
+                     fecState = (message[11] & 0xF0) >> 4;
+                     break;
+
+                case FITNESS_EQUIPMENT_TRAINER_TORQUE_PAGE:
+                     //based on "ANT+ Device Profile Fitness Equipment" rev 4.1 p 61: 6.6.8 Data page 0x20 - Specific trainer torque data
+                     fecPage0x20EventCount = message[5];
+                     wheelRevolutions = message[6];                 // increments with each wheel revolution, rollover 256 revolutions
+                     wheelAccumulatedPeriod = message[7];           // accumulated wheel period ; unit: 1/2048s ; rollover 32s
+                     wheelAccumulatedPeriod |= message[8] << 8;
+                     accumulatedTorque = message[9];                // accumulated torque ; unit: 1/32Nm ; rollover 2048Nm
+                     accumulatedTorque |= message[10] << 8;
+                     fecState = (message[11] & 0xF0) >> 4;
+                     break;
+
+                case FITNESS_EQUIPMENT_TRAINER_CAPABILITIES_PAGE:
+                     fecMaxResistance         = message[9];
+                     fecMaxResistance        |= (message[10]) << 8;
+                     fecCapabilities          = message[11];
+                     break;
+
+                case FITNESS_EQUIPMENT_COMMAND_STATUS_PAGE:
+                    // Acknowledge from device
+                    fecLastCommandReceived = message[5];
+                    fecLastCommandSeq      = message[6];
+                    fecLastCommandStatus   = message[7];
+                    switch (fecLastCommandReceived)
+                    {
+                    case FITNESS_EQUIPMENT_BASIC_RESISTANCE_ID:
+                        fecSetResistanceAck = (double) message[11] * 0.5;
+                        break;
+                    case FITNESS_EQUIPMENT_TARGET_POWER_ID:
+                        fecSetTargetPowerAck   = message[10] >> 2;   // device power step is 0.25W, GC consider 1W accuracy only
+                        fecSetTargetPowerAck  |= (message[11]) << 6;
+                        break;
+                    case FITNESS_EQUIPMENT_WIND_RESISTANCE_ID:
+                        fecSetWindResistanceAck = (double) message[9] * 0.01;
+                        fecSetWindSpeedAck      = message[10];
+                        fecSetDraftingFactorAck = message[11];
+                        break;
+                    case FITNESS_EQUIPMENT_TRACK_RESISTANCE_ID:
+                        fecSetGradeAck          = (double) ((int16_t) message[9] | ((int16_t) message[10] << 8)) * 0.01;
+                        fecSetRollResistanceAck = (double) message[11] * 0.00005;
+                        break;
+                    }
+                    break;
+
+                }
+                break;
+
             case ANTChannel::CHANNEL_TYPE_TACX_VORTEX:
             {
                 const uint8_t* const payload = message + 4;
@@ -649,6 +725,12 @@ void ANTMessage::init()
     pedalPower = 0;
     leftTorqueEffectiveness = rightTorqueEffectiveness = 0;
     leftOrCombinedPedalSmoothness = rightPedalSmoothness = 0;
+    fecSpeed = fecInstantPower = fecAccumulatedPower = 0;
+    fecRawDistance = fecCadence = fecPage0x19EventCount = fecPage0x20EventCount = 0;
+    fecPowerCalibRequired = fecResisCalibRequired = fecUserConfigRequired = false;
+    fecPowerOverLimits = fecState = fecHRSource = 0;
+    fecDistanceCapability = fecSpeedIsVirtual = false;
+    fecEqtType = 0;
 }
 
 ANTMessage ANTMessage::resetSystem()
@@ -728,7 +810,7 @@ ANTMessage ANTMessage::requestCalibrate(const unsigned char channel)
 ANTMessage ANTMessage::ANTMessage::boostSignal(const unsigned char channel)
 {
     // [A4][02][6A][XX][57][9B]
-    return ANTMessage(2, 0x6A, channel, 0x57); 
+    return ANTMessage(2, 0x6A, channel, 0x57);
 }
 
 ANTMessage ANTMessage::open(const unsigned char channel)
@@ -886,3 +968,61 @@ ANTMessage ANTMessage::kickrInitSpindown(const unsigned char channel, ushort usD
     return ANTMessage(5, ANT_BROADCAST_DATA, channel, // broadcast
            KICKR_READ_MODE, (unsigned char)usDeviceId, (unsigned char)(usDeviceId>>8)); // preamble
 }
+
+ANTMessage ANTMessage::fecSetResistance(const uint8_t channel, const uint8_t resistance) // unit is 0.5% => 200 = 100%
+{
+    return ANTMessage(9, ANT_ACK_DATA, channel,
+                      FITNESS_EQUIPMENT_BASIC_RESISTANCE_ID,
+                      0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, resistance);
+}
+
+ANTMessage ANTMessage::fecSetTargetPower(const uint8_t channel, const uint16_t targetPower)
+{
+    // unit is 0.25W, but targetPower are full watts and theres no trainer with that precision anyway
+    uint16_t powerValue = targetPower * 4;
+    return ANTMessage(9, ANT_ACK_DATA, channel,
+                      FITNESS_EQUIPMENT_TARGET_POWER_ID,
+                      0xFF, 0xFF, 0xFF, 0xFF, 0xFF, (uint8_t)(powerValue & 0xFF), (uint8_t)(powerValue >> 8));
+}
+
+ANTMessage ANTMessage::fecSetWindResistance(const uint8_t channel, const double windResistance, const uint8_t windSpeed, const uint8_t draftingFactor)
+                                                                    //     0.00 < kg/m < 1.86         -127 < kph < + 127               0 < % < 100%
+{
+    return ANTMessage(9, ANT_ACK_DATA, channel,
+                      FITNESS_EQUIPMENT_WIND_RESISTANCE_ID,
+                      0xFF, 0xFF, 0xFF, 0xFF, (uint8_t) (windResistance * 100), windSpeed, draftingFactor);
+}
+
+ANTMessage ANTMessage::fecSetTrackResistance(const uint8_t channel, const double grade, const double rollingResistance)
+                                                             //     -200 < % < +200%                   0.0 <  < 0.0127
+{
+    uint16_t rawGradeValue = (uint16_t) ((grade+200.0) * 100.0);
+    uint8_t  rollingResistanceValue = rollingResistance / 0.00005;
+    return ANTMessage(9, ANT_ACK_DATA, channel,
+                      FITNESS_EQUIPMENT_TRACK_RESISTANCE_ID,
+                      0xFF, 0xFF, 0xFF, 0xFF, (uint8_t)(rawGradeValue & 0xFF), (uint8_t)(rawGradeValue >> 8), rollingResistanceValue);
+}
+
+ANTMessage ANTMessage::fecRequestCapabilities(const uint8_t channel)
+{
+    // based on ANT+ Common Pages, Rev 2.4 p 14: 6.2  Common Data Page 70: Request Data Page
+    return ANTMessage(9, ANT_ACK_DATA, channel,
+                      0x46,        // data page request
+                      0xFF, 0xFF,  // reserved
+                      0xFF, 0xFF,  // descriptors
+                      0x01,        // requested transmission response (1 time only)
+                      0x36,        // requested page
+                      0x01);       // request data page
+}
+
+ANTMessage ANTMessage::fecRequestCommandStatus(const uint8_t channel, const uint8_t page)
+{
+    // based on ANT+ Common Pages, Rev 2.4 p 14: 6.2  Common Data Page 70: Request Data Page
+    return ANTMessage(9, ANT_ACK_DATA, channel,
+                      0x46,        // data page request
+                      0xFF, 0xFF,  // reserved
+                      0xFF, 0xFF,  // descriptors
+                      0x01,        // requested transmission response (1 time only)
+                      page,        // requested page (0x30 = resistance, 0x31 = power, 0x32 = wind, 0x33 = slope/track)
+                      0x01);       // request data page
+}
