@@ -32,6 +32,9 @@
 #include "FortiusController.h"
 #endif
 #include "ComputrainerController.h"
+#if QT_VERSION >= 0x050000
+#include "MonarkController.h"
+#endif
 #include "ANTlocalController.h"
 #include "ANTChannel.h"
 #include "NullController.h"
@@ -171,6 +174,7 @@ class AddPair : public QWizardPage
         QTreeWidget *channelWidget;
         QSignalMapper *signalMapper;
         QTimer updateValues;
+        QString cyclist;
 };
 
 class AddPairBTLE : public QWizardPage
@@ -199,6 +203,8 @@ class AddPairBTLE : public QWizardPage
         QTreeWidget *channelWidget;
         QSignalMapper *signalMapper;
         QTimer updateValues;
+        QString cyclist;
+
 };
 
 class AddFinal : public QWizardPage
