@@ -631,7 +631,7 @@ AddIntervalDialog::findFirsts(bool typeTime, const RideFile *ride, double window
 
     // ride is shorter than the window size!
     if (typeTime && windowSize > ride->dataPoints().last()->secs + secsDelta) return;
-    else if (windowSize > ride->dataPoints().last()->km*1000) return;
+    else if (!typeTime && windowSize > ride->dataPoints().last()->km*1000) return;
 
     double rest = 0;
     // We're looking for intervals with durations in [windowSizeSecs, windowSizeSecs + secsDelta).
