@@ -27,7 +27,7 @@
 #include "Units.h"
 #include "Colors.h"
 
-#include <math.h>
+#include <cmath>
 #include <qwt_series_data.h>
 #include <qwt_legend.h>
 #include <qwt_plot_curve.h>
@@ -89,26 +89,8 @@ IntervalItem *IntervalAerolabData::intervalNum
     int number
 ) const
 {
-    int                    highlighted  = 0;
-    const QTreeWidgetItem *allIntervals = context->athlete->allIntervalItems();
-    for ( int ii = 0; ii < allIntervals->childCount(); ii++)
-
-    {
-        IntervalItem *current = (IntervalItem *) allIntervals->child( ii );
-
-        if ( current == NULL)
-        {
-            return NULL;
-        }
-        if ( current->isSelected() == true )
-        {
-            ++highlighted;
-        }
-        if ( highlighted == number )
-        {
-            return current;
-        }
-    }
+    if (number >= 0 && aerolab->rideItem->intervalsSelected().count() > number)
+        return aerolab->rideItem->intervalsSelected().at(number);
 
     return NULL;
 }
@@ -118,24 +100,7 @@ IntervalItem *IntervalAerolabData::intervalNum
 // ------------------------------------------------------------------------------------------------------------
 int IntervalAerolabData::intervalCount() const
 {
-    int highlighted = 0;
-
-    if ( context->athlete->allIntervalItems() != NULL )
-    {
-        const QTreeWidgetItem *allIntervals = context->athlete->allIntervalItems();
-        for ( int ii = 0; ii < allIntervals->childCount(); ii++)
-        {
-            IntervalItem *current = (IntervalItem *) allIntervals->child( ii );
-            if ( current != NULL )
-            {
-                if ( current->isSelected() == true )
-                {
-                    ++highlighted;
-                }
-            }
-        }
-    }
-    return highlighted;
+    return aerolab->rideItem->intervalsSelected().count();
 }
 /*
  * INTERVAL HIGHLIGHTING CURVE
@@ -243,6 +208,7 @@ Aerolab::Aerolab(
   rho       = 1.236;
   eta       = 1.0;
   eoffset   = 0.0;
+  constantAlt = false;
 
   insertLegend(new QwtLegend(), QwtPlot::BottomLegend);
   setCanvasBackground(Qt::white);
@@ -272,12 +238,12 @@ Aerolab::Aerolab(
   grid->enableX(false);
   grid->attach(this);
 
-  configChanged();
+  configChanged(CONFIG_APPEARANCE);
 }
 
 
 void
-Aerolab::configChanged()
+Aerolab::configChanged(qint32)
 {
 
   // set colors
@@ -389,7 +355,7 @@ Aerolab::setData(RideItem *_rideItem, bool new_zoom) {
       double f     = 0.0;
       double a     = 0.0;
 
-      // Use km data insteed of formula for file with a stop (gap).
+      // Use km data instead of formula for file with a stop (gap).
       //d += v * dt;
       //distanceArray[arrayLength] = d/1000;
 
@@ -406,10 +372,11 @@ Aerolab::setData(RideItem *_rideItem, bool new_zoom) {
 
       f *= eta; // adjust for drivetrain efficiency if using a crank-based meter
       double s   = slope( f, a, m, crr, cda, rho, headwind );
-      double de  = s * v * dt;
+      double de  = s * v * dt * (context->athlete->useMetricUnits ? 1 : FEET_PER_METER);
 
       e += de;
       t += dt;
+
       veArray[arrayLength] = e;
 
       vlast = v;
@@ -759,9 +726,9 @@ void Aerolab::refreshIntervalMarkers()
     QRegExp wkoAuto("^(Peak *[0-9]*(s|min)|Entire workout|Find #[0-9]*) *\\([^)]*\\)$");
     if ( rideItem->ride() )
     {
-        foreach(const RideFileInterval &interval, rideItem->ride()->intervals()) {
+        foreach(IntervalItem *interval, rideItem->intervals()) {
             // skip WKO autogenerated peak intervals
-            if (wkoAuto.exactMatch(interval.name))
+            if (wkoAuto.exactMatch(interval->name))
                 continue;
             QwtPlotMarker *mrk = new QwtPlotMarker;
             d_mrk.append(mrk);
@@ -769,14 +736,14 @@ void Aerolab::refreshIntervalMarkers()
             mrk->setLineStyle(QwtPlotMarker::VLine);
             mrk->setLabelAlignment(Qt::AlignRight | Qt::AlignTop);
             mrk->setLinePen(QPen(GColor(CPLOTMARKER), 0, Qt::DashDotLine));
-            QwtText text(interval.name);
+            QwtText text(interval->name);
             text.setFont(QFont("Helvetica", 10, QFont::Bold));
             text.setColor(GColor(CPLOTMARKER));
             if (!bydist)
-                mrk->setValue(interval.start / 60.0, 0.0);
+                mrk->setValue(interval->start / 60.0, 0.0);
             else
                 mrk->setValue((context->athlete->useMetricUnits ? 1 : MILES_PER_KM) *
-                                rideItem->ride()->timeToDistance(interval.start), 0.0);
+                                interval->startKM, 0.0);
             mrk->setLabel(text);
         }
     }
@@ -905,7 +872,7 @@ QString Aerolab::estimateCdACrr(RideItem *rideItem)
             errMsg = tr("Altitude and Power data must be present");
         }
     } else {
-        errMsg = tr("No ride selected");
+        errMsg = tr("No activity selected");
     }
     return errMsg;
 }
