@@ -45,10 +45,11 @@
 #include <QStyleFactory>
 #include <QStyle>
 
-
+#include "UserData.h"
 #include "RideFile.h"
 
 class QwtPlotCurve;
+class QwtPlotGappedCurve;
 class QwtPlotIntervalCurve;
 class QwtPlotGrid;
 class QwtPlotMarker;
@@ -178,7 +179,7 @@ class CurveColors : public QObject
                              static_cast<QwtPlotSeriesItem*>(item)->isVisible());
 
                 QwtScaleWidget *x = plot->axisWidget(static_cast<QwtPlotSeriesItem*>(item)->yAxis());
-                colors.insert(x, x->palette());
+                if (x) colors.insert(x, x->palette());
 
                 QwtPlotCurve *curve = static_cast<QwtPlotCurve*>(item);
 
@@ -196,7 +197,7 @@ class CurveColors : public QObject
                              static_cast<QwtPlotSeriesItem*>(item)->isVisible());
 
                 QwtScaleWidget *x = plot->axisWidget(static_cast<QwtPlotSeriesItem*>(item)->yAxis());
-                colors.insert(x, x->palette());
+                if (x) colors.insert(x, x->palette());
 
             }
         }
@@ -341,6 +342,15 @@ class CurveColors : public QObject
 
 };
 
+// Plotting user data
+struct UserObject {
+    QString name, units;
+    QVector<double> array;
+    QVector<double> smooth;
+    QwtPlotGappedCurve    *curve;
+    QColor          color;
+};
+
 class AllPlot;
 class MergeAdjust;
 class AllPlotObject : public QObject
@@ -353,7 +363,7 @@ class AllPlotObject : public QObject
 
     public:
 
-    AllPlotObject(AllPlot*); // construct associate with a plot
+    AllPlotObject(AllPlot*, QList<UserData*>); // construct associate with a plot
     ~AllPlotObject(); // delete and disassociate from a plot
 
     void setVisible(bool); // show or hide objects
@@ -515,6 +525,10 @@ class AllPlotObject : public QObject
     QVector<QwtIntervalSample> smoothRPPP;
     QVector<QwtIntervalSample> smoothRelSpeed;
 
+    // setup as copy from user data
+    void setUserData(QList<UserData*>); // reset below to reflect current
+    QList<UserObject> U;
+
     // highlighting intervals
     QwtPlotCurve *intervalHighlighterCurve,  // highlight selected intervals on the Plot
                  *intervalHoverCurve;
@@ -543,8 +557,8 @@ class AllPlot : public QwtPlot
         bool eventFilter(QObject *object, QEvent *e);
 
         // set the curve data e.g. when a ride is selected
-        void setDataFromRide(RideItem *_rideItem);
-        void setDataFromRideFile(RideFile *ride, AllPlotObject *object); // when plotting lots of rides on fullPlot
+        void setDataFromRide(RideItem *_rideItem, QList<UserData*>);
+        void setDataFromRideFile(RideFile *ride, AllPlotObject *object, QList<UserData*>); // when plotting lots of rides on fullPlot
         void setDataFromPlot(AllPlot *plot, int startidx, int stopidx);
         void setDataFromPlot(AllPlot *plot); // used for single series plotting
         void setDataFromPlots(QList<AllPlot*>); // user for single series comparing
