@@ -177,6 +177,9 @@ struct Bin2FileReaderState
             nm = read_bytes(2, bytes_read, sum);
             kph = read_bytes(2, bytes_read, sum);
             alt = read_bytes(2, bytes_read, sum); // todo this value is signed
+            if (data_version>=9) {
+                alt = (alt-5000)/10.0; // todo this value is signed
+            }
             temp = read_bytes(2, bytes_read, sum); // °C × 10 todo this value is signed
             lat = read_bytes(4, bytes_read, sum); // todo this value is signed
             lng = read_bytes(4, bytes_read, sum); // todo this value is signed
@@ -292,6 +295,8 @@ struct Bin2FileReaderState
     void read_username(int *bytes_read = NULL, int *sum = NULL)
     {
         QString name = read_text(20, bytes_read, sum);
+        if (BIN2_DEBUG)
+            qDebug() << "User" << name;
         //deviceInfo += QString("User : %1\n").arg(name);
     }
 
@@ -302,6 +307,9 @@ struct Bin2FileReaderState
         int smartbelt_A = read_bytes(2, bytes_read, sum);
         int smartbelt_B = read_bytes(2, bytes_read, sum);
         int smartbelt_C = read_bytes(2, bytes_read, sum);
+
+        if (BIN2_DEBUG)
+            qDebug() << "Smartbelt" << smartbelt_A << smartbelt_B << smartbelt_C;
         deviceInfo += QString("Smartbelt %1-%2-%3\n").arg(smartbelt_A).arg(smartbelt_B).arg(smartbelt_C);
 
         read_bytes(42, bytes_read, sum);
@@ -340,6 +348,8 @@ struct Bin2FileReaderState
                 else
                     device_type_str = QString("ANT %1 Id").arg(device_type);
 
+                if (BIN2_DEBUG)
+                    qDebug() << "ANT" << device_type_str << id << text;
                 deviceInfo += QString("%1 %2 %3\n").arg(device_type_str).arg(id).arg(text);
             }
         }
@@ -348,6 +358,9 @@ struct Bin2FileReaderState
     // pages
     int read_summary_page()
     {
+        if (BIN2_DEBUG)
+            qDebug() << "read_summary_page()";
+
         int sum = 0;
         int bytes_read = 0;
 
@@ -361,6 +374,9 @@ struct Bin2FileReaderState
             uint16_t length = read_bytes(2, &bytes_read, &sum);
             uint16_t page_number = read_bytes(2, &bytes_read, &sum); // Page #
 
+            if (BIN2_DEBUG)
+                qDebug() << "page_number" << page_number;
+
             if (page_number == 0) {
                 // Page #0
                 if (jouleGPS_GPSPlus) {
@@ -536,6 +552,9 @@ struct Bin2FileReaderState
 
     int read_version()
     {
+        if (BIN2_DEBUG)
+            qDebug() << "read_version()";
+
         int sum = 0;
         int bytes_read = 0;
 
@@ -544,7 +563,9 @@ struct Bin2FileReaderState
 
         if (header == START && command == UNIT_VERSION)
         {
-            read_bytes(2, &bytes_read, &sum); // length
+            int length = read_bytes(2, &bytes_read, &sum); // length
+            if (BIN2_DEBUG)
+                qDebug() << "length" << length;
 
             int major_version = read_bytes(1, &bytes_read, &sum);
             int minor_version = read_bytes(2, &bytes_read, &sum);
@@ -562,7 +583,6 @@ struct Bin2FileReaderState
                     jouleGPS = false;
             }
 
-
             if (jouleGPS_GPSPlus)
                 data_version = read_bytes(2, &bytes_read, &sum);
 
@@ -574,10 +594,13 @@ struct Bin2FileReaderState
             else
                 deviceInfo += "Joule";
 
+            if (BIN2_DEBUG)
+                qDebug() << "Version" << version;
+
             deviceInfo += QString(" Version %1\n").arg(version);
 
-
-
+            if (jouleGPS_GPSPlus && !jouleGPS)
+                read_bytes(5, &bytes_read, &sum);
 
             read_bytes(1, &bytes_read, &sum); // checksum
         }
@@ -586,16 +609,23 @@ struct Bin2FileReaderState
 
     int read_system_info()
     {
+        if (BIN2_DEBUG)
+            qDebug() << "read_system_info()";
+
         int sum = 0;
         int bytes_read = 0;
 
         uint16_t header = read_bytes(2, &bytes_read, &sum); // Always (0x10-0x02)
         uint16_t command = read_bytes(2, &bytes_read, &sum);
 
+        if (BIN2_DEBUG)
+            qDebug() << "header" << header << "command" << command;
 
         if (header == START && command == SYSTEM_INFO)
         {
-            read_bytes(2, &bytes_read, &sum); // length
+            int length = read_bytes(2, &bytes_read, &sum); // length
+            if (BIN2_DEBUG)
+                qDebug() << "length" << length;
 
             if (jouleGPS_GPSPlus)
                 read_bytes(52, &bytes_read, &sum);
@@ -609,6 +639,10 @@ struct Bin2FileReaderState
                 read_bytes(34, &bytes_read, &sum);
 
             read_bytes(1, &bytes_read, &sum); // checksum
+        } else
+        {
+            if (BIN2_DEBUG)
+                qDebug() << "unknown header or command";
         }
         return bytes_read;
     }
