@@ -18,10 +18,12 @@
 
 
 #include "ExtendedCriticalPower.h"
+#include "Statistic.h"
 #include "Colors.h"
 #include "Settings.h"
 #include "RideFileCache.h"
 #include <qwt_symbol.h>
+#include <qwt_color_map.h>
 
 #include <QtGui>
 #include <QMessageBox>
@@ -34,10 +36,10 @@ ExtendedCriticalPower::~ExtendedCriticalPower()
 {
 }
 
-Model_eCP
+TestModel
 ExtendedCriticalPower::deriveExtendedCP_2_3_Parameters(RideFileCache *bests, RideFile::SeriesType series, double sanI1, double sanI2, double anI1, double anI2, double aeI1, double aeI2, double laeI1, double laeI2)
 {
-    Model_eCP model;
+    TestModel model;
     model.version = "eCP v2.3";
 
     // bounds on anaerobic interval in minutes
@@ -56,7 +58,7 @@ ExtendedCriticalPower::deriveExtendedCP_2_3_Parameters(RideFileCache *bests, Rid
     const double t7 = laeI1;
     const double t8 = laeI2;
 
-    // bounds of these time valus in the data
+    // bounds of these time values in the data
     int i1, i2, i3, i4, i5, i6, i7, i8;
 
     // find the indexes associated with the bounds
@@ -254,7 +256,7 @@ ExtendedCriticalPower::deriveExtendedCP_2_3_Parameters(RideFileCache *bests, Rid
 }
 
 QwtPlotCurve*
-ExtendedCriticalPower::getPlotCurveForExtendedCP_2_3(Model_eCP model)
+ExtendedCriticalPower::getPlotCurveForExtendedCP_2_3(TestModel model)
 {
     const int extendedCurve2_points = 1000;
 
@@ -282,10 +284,10 @@ ExtendedCriticalPower::getPlotCurveForExtendedCP_2_3(Model_eCP model)
     return extendedCPCurve2;
 }
 
-Model_eCP
+TestModel
 ExtendedCriticalPower::deriveExtendedCP_4_3_Parameters(bool usebest, RideFileCache *bests, RideFile::SeriesType series, double sanI1, double sanI2, double anI1, double anI2, double aeI1, double aeI2, double laeI1, double laeI2)
 {
-    Model_eCP model;
+    TestModel model;
     model.version = "eCP v4.3";
 
     // bounds on anaerobic interval in minutes
@@ -304,7 +306,7 @@ ExtendedCriticalPower::deriveExtendedCP_4_3_Parameters(bool usebest, RideFileCac
     const double t7 = laeI1;
     const double t8 = laeI2;
 
-    // bounds of these time valus in the data
+    // bounds of these time values in the data
     int i1, i2, i3, i4, i5, i6, i7, i8;
 
     // find the indexes associated with the bounds
@@ -530,10 +532,10 @@ ExtendedCriticalPower::deriveExtendedCP_4_3_Parameters(bool usebest, RideFileCac
     return model;
 }
 
-Model_eCP
+TestModel
 ExtendedCriticalPower::deriveExtendedCP_4_3_ParametersForBest(double best5s, double best1min, double best5min, double best1hour)
 {
-    Model_eCP model;
+    TestModel model;
 
     // initial estimate of tau
     model.paa = 300;
@@ -609,7 +611,7 @@ ExtendedCriticalPower::deriveExtendedCP_4_3_ParametersForBest(double best5s, dou
 }
 
 QwtPlotCurve*
-ExtendedCriticalPower::getPlotCurveForExtendedCP_4_3(Model_eCP model)
+ExtendedCriticalPower::getPlotCurveForExtendedCP_4_3(TestModel model)
 {
     //qDebug() <<"getPlotCurveForExtendedCP_4_3()";
     //qDebug() <<"Model eCP(4.3) " << "paa" << model.paa  << "ecp" << model.ecp << "etau" << model.etau << "paa_dec" << model.paa_dec << "ecp_del" << model.ecp_del  << "ecp_dec" << model.ecp_dec ;
@@ -641,7 +643,7 @@ ExtendedCriticalPower::getPlotCurveForExtendedCP_4_3(Model_eCP model)
 }
 
 QwtPlotCurve*
-ExtendedCriticalPower::getPlotLevelForExtendedCP_4_3(Model_eCP model)
+ExtendedCriticalPower::getPlotLevelForExtendedCP_4_3(TestModel model)
 {
     const int extendedCurve2_points = 20;
 
@@ -705,7 +707,7 @@ ExtendedCriticalPower::getPlotLevelForExtendedCP_4_3(Model_eCP model)
 }
 
 QwtPlotMarker*
-ExtendedCriticalPower::getPlotMarkerForExtendedCP(Model_eCP model)
+ExtendedCriticalPower::getPlotMarkerForExtendedCP(TestModel model)
 {
     QwtPlotMarker* extendedCurveTitle2 = new QwtPlotMarker();
     QString extendedCurve2_title;
@@ -719,7 +721,7 @@ ExtendedCriticalPower::getPlotMarkerForExtendedCP(Model_eCP model)
 }
 
 QwtPlotCurve*
-ExtendedCriticalPower::getPlotCurveForExtendedCP_4_3_WSecond(Model_eCP model)
+ExtendedCriticalPower::getPlotCurveForExtendedCP_4_3_WSecond(TestModel model)
 {
     const int extendedCurve2_points = 1000;
 
@@ -748,7 +750,7 @@ ExtendedCriticalPower::getPlotCurveForExtendedCP_4_3_WSecond(Model_eCP model)
 }
 
 QwtPlotCurve*
-ExtendedCriticalPower::getPlotCurveForExtendedCP_4_3_WPrime(Model_eCP model)
+ExtendedCriticalPower::getPlotCurveForExtendedCP_4_3_WPrime(TestModel model)
 {
     const int extendedCurve2_points = 1000;
 
@@ -777,7 +779,7 @@ ExtendedCriticalPower::getPlotCurveForExtendedCP_4_3_WPrime(Model_eCP model)
 }
 
 QwtPlotCurve*
-ExtendedCriticalPower::getPlotCurveForExtendedCP_4_3_CP(Model_eCP model)
+ExtendedCriticalPower::getPlotCurveForExtendedCP_4_3_CP(TestModel model)
 {
     const int extendedCurve2_points = 1000;
 
@@ -808,7 +810,7 @@ ExtendedCriticalPower::getPlotCurveForExtendedCP_4_3_CP(Model_eCP model)
 
 
 QwtPlotCurve*
-ExtendedCriticalPower::getPlotCurveForExtendedCP_4_3_WPrime_CP(Model_eCP model)
+ExtendedCriticalPower::getPlotCurveForExtendedCP_4_3_WPrime_CP(TestModel model)
 {
     const int extendedCurve2_points = 1000;
 
@@ -836,29 +838,29 @@ ExtendedCriticalPower::getPlotCurveForExtendedCP_4_3_WPrime_CP(Model_eCP model)
     return extendedCPCurve2;
 }
 
-Model_eCP
+TestModel
 ExtendedCriticalPower::deriveExtendedCP_5_3_Parameters(bool usebest, RideFileCache *bests, RideFile::SeriesType series, double sanI1, double sanI2, double anI1, double anI2, double aeI1, double aeI2, double laeI1, double laeI2)
 {
-    Model_eCP model;
+    TestModel model;
     model.version = "eCP v5.3";
 
     // bounds on anaerobic interval in minutes
-    const double t1 = sanI1;
-    const double t2 = sanI2;
+    const double t1 = sanI1 / 60.0;
+    const double t2 = sanI2 / 60.0;
 
     // bounds on anaerobic interval in minutes
-    const double t3 = anI1;
-    const double t4 = anI2;
+    const double t3 = anI1 / 60.0;
+    const double t4 = anI2 / 60.0;
 
     // bounds on aerobic interval in minutes
-    const double t5 = aeI1;
-    const double t6 = aeI2;
+    const double t5 = aeI1 / 60.0;
+    const double t6 = aeI2 / 60.0;
 
     // bounds on long aerobic interval in minutes
-    const double t7 = laeI1;
-    const double t8 = laeI2;
+    const double t7 = laeI1 / 60.0;
+    const double t8 = laeI2 / 60.0;
 
-    // bounds of these time valus in the data
+    // bounds of these time values in the data
     int i1, i2, i3, i4, i5, i6, i7, i8;
 
     // find the indexes associated with the bounds
@@ -951,6 +953,8 @@ ExtendedCriticalPower::deriveExtendedCP_5_3_Parameters(bool usebest, RideFileCac
 
     // loop to convergence
     int iteration = 0;
+
+    qDebug() << "start iteration";
     do {
         if (iteration ++ > max_loops) {
             qDebug()<<"Maximum number of loops exceeded in ecp2 model";
@@ -1075,21 +1079,21 @@ ExtendedCriticalPower::deriveExtendedCP_5_3_Parameters(bool usebest, RideFileCac
              (fabs(model.ecp_dec - ecp_dec_prev) > ecp_dec_delta_max)
             );
 
-    //qDebug() << iteration << "iterations";
+    qDebug() << iteration << "iterations";
 
     model.pMax = model.paa*(1.20-0.20*exp(-1*(1/60.0)))*exp(model.paa_dec*(1/60.0)) + model.ecp * (1-exp(model.tau_del*(1/60.0))) * (1-exp(model.ecp_del*(1/60.0))) * (1+model.ecp_dec*exp(model.ecp_dec_del/(1/60.0))) * ( 1 + model.etau/(1/60.0));
     model.mmp60 = model.paa*(1.20-0.20*exp(-1*60.0))*exp(model.paa_dec*(60.0)) + model.ecp * (1-exp(model.tau_del*(60.0))) * (1-exp(model.ecp_del*60.0)) * (1+model.ecp_dec*exp(model.ecp_dec_del/60.0)) * ( 1 + model.etau/(60.0));
 
-    qDebug() <<"eCP(5.3) " << "paa" << model.paa  << "ecp" << model.ecp << "etau" << model.etau << "paa_dec" << model.paa_dec << "ecp_del" << model.ecp_del << "ecp_dec" << model.ecp_dec << "ecp_dec_del" << model.ecp_dec_del;
-    qDebug() <<"eCP(5.3) " << "pmax" << model.pMax << "mmp60" << model.mmp60;
+    qDebug() <<"-> eCP(5.3) " << "paa" << model.paa  << "ecp" << model.ecp << "etau" << model.etau << "paa_dec" << model.paa_dec << "ecp_del" << model.ecp_del << "ecp_dec" << model.ecp_dec << "ecp_dec_del" << model.ecp_dec_del;
+    qDebug() <<"-> eCP(5.3) " << "pmax" << model.pMax << "mmp60" << model.mmp60;
 
     return model;
 }
 
-Model_eCP
+TestModel
 ExtendedCriticalPower::deriveExtendedCP_5_3_ParametersForBest(double best5s, double best1min, double best5min, double best1hour)
 {
-    Model_eCP model;
+    TestModel model;
     model.version = "eCP v5.3";
 
     // initial estimate of tau
@@ -1158,7 +1162,7 @@ ExtendedCriticalPower::deriveExtendedCP_5_3_ParametersForBest(double best5s, dou
 }
 
 QwtPlotCurve*
-ExtendedCriticalPower::getPlotCurveForExtendedCP_5_3(Model_eCP model)
+ExtendedCriticalPower::getPlotCurveForExtendedCP_5_3(TestModel model)
 {
     const int extendedCurve2_points = 1000;
 
@@ -1177,7 +1181,7 @@ ExtendedCriticalPower::getPlotCurveForExtendedCP_5_3(Model_eCP model)
     QwtPlotCurve *extendedCPCurve2 = new QwtPlotCurve("eCP_5_3");
     if (appsettings->value(NULL, GC_ANTIALIAS, true).toBool() == true)
         extendedCPCurve2->setRenderHint(QwtPlotItem::RenderAntialiased);
-    QPen e2pen(GColor(CCP)); // Qt::cyan
+    QPen e2pen(Qt::cyan); // Qt::cyan GColor(CCP)
     e2pen.setWidth(1);
     e2pen.setStyle(Qt::DashLine);
     extendedCPCurve2->setPen(e2pen);
@@ -1187,7 +1191,7 @@ ExtendedCriticalPower::getPlotCurveForExtendedCP_5_3(Model_eCP model)
 }
 
 QwtPlotCurve*
-ExtendedCriticalPower::getPlotLevelForExtendedCP_5_3(Model_eCP model)
+ExtendedCriticalPower::getPlotLevelForExtendedCP_5_3(TestModel model)
 {
     const int extendedCurve2_points = 1000;
 
@@ -1216,7 +1220,7 @@ ExtendedCriticalPower::getPlotLevelForExtendedCP_5_3(Model_eCP model)
 }
 
 QwtPlotIntervalCurve*
-ExtendedCriticalPower::getPlotCurveForExtendedCP_5_3_WSecond(Model_eCP model, bool)
+ExtendedCriticalPower::getPlotCurveForExtendedCP_5_3_WSecond(TestModel model, bool stacked, bool inversed)
 {
     const int extendedCurve2_points = 1000;
 
@@ -1230,8 +1234,11 @@ ExtendedCriticalPower::getPlotCurveForExtendedCP_5_3_WSecond(Model_eCP model, bo
         double t = pow(tmax, x) * pow(tmin, 1-x);
 
         double power_wsecond = model.paa*(1.20-0.20*exp(-1*t))*exp(model.paa_dec*(t));
+        double power_wprime = model.ecp * (1-exp(model.tau_del*t)) * (1-exp(model.ecp_del*t)) * (1+model.ecp_dec*exp(model.ecp_dec_del/t)) * ( model.etau/(t) );
+        double power_cp = model.ecp * (1-exp(model.tau_del*t)) * (1-exp(model.ecp_del*t)) * (1+model.ecp_dec*exp(model.ecp_dec_del/t)) * ( 1 );
 
-        extended_cp_curve_power[i] = QwtIntervalSample(t, 0, power_wsecond);
+
+        extended_cp_curve_power[i] = QwtIntervalSample(t, (stacked?(inversed?power_cp+power_wprime:0):0), (stacked?(inversed?power_cp+power_wprime+power_wsecond:power_wsecond):power_wsecond));
     }
 
     QwtPlotIntervalCurve *extendedCPCurve = new QwtPlotIntervalCurve("eCP_5_3_WSecond");
@@ -1258,7 +1265,7 @@ ExtendedCriticalPower::getPlotCurveForExtendedCP_5_3_WSecond(Model_eCP model, bo
 }
 
 QwtPlotIntervalCurve*
-ExtendedCriticalPower::getPlotCurveForExtendedCP_5_3_WPrime(Model_eCP model, bool stacked)
+ExtendedCriticalPower::getPlotCurveForExtendedCP_5_3_WPrime(TestModel model, bool stacked, bool inversed)
 {
     const int extendedCurve2_points = 1000;
 
@@ -1273,8 +1280,9 @@ ExtendedCriticalPower::getPlotCurveForExtendedCP_5_3_WPrime(Model_eCP model, boo
 
         double power_wsecond = model.paa*(1.20-0.20*exp(-1*t))*exp(model.paa_dec*(t));
         double power_wprime = model.ecp * (1-exp(model.tau_del*t)) * (1-exp(model.ecp_del*t)) * (1+model.ecp_dec*exp(model.ecp_dec_del/t)) * ( model.etau/(t) );
+        double power_cp = model.ecp * (1-exp(model.tau_del*t)) * (1-exp(model.ecp_del*t)) * (1+model.ecp_dec*exp(model.ecp_dec_del/t)) * ( 1 );
 
-        extended_cp_curve_power[i] = QwtIntervalSample(t, (stacked?power_wsecond:0), (stacked?power_wprime + power_wsecond:power_wprime));
+        extended_cp_curve_power[i] = QwtIntervalSample(t, (stacked?(inversed?power_cp:power_wsecond):0), (stacked?(inversed?power_wprime + power_cp: power_wprime + power_wsecond):power_wprime));
     }
 
     QwtPlotIntervalCurve *extendedCPCurve = new QwtPlotIntervalCurve("eCP_5_3_WPrime");
@@ -1302,7 +1310,7 @@ ExtendedCriticalPower::getPlotCurveForExtendedCP_5_3_WPrime(Model_eCP model, boo
 }
 
 QwtPlotIntervalCurve*
-ExtendedCriticalPower::getPlotCurveForExtendedCP_5_3_CP(Model_eCP model, bool stacked)
+ExtendedCriticalPower::getPlotCurveForExtendedCP_5_3_CP(TestModel model, bool stacked, bool inversed)
 {
     const int extendedCurve2_points = 1000;
 
@@ -1319,7 +1327,7 @@ ExtendedCriticalPower::getPlotCurveForExtendedCP_5_3_CP(Model_eCP model, bool st
         double power_wprime = model.ecp * (1-exp(model.tau_del*t)) * (1-exp(model.ecp_del*t)) * (1+model.ecp_dec*exp(model.ecp_dec_del/t)) * ( model.etau/(t) );
         double power_cp = model.ecp * (1-exp(model.tau_del*t)) * (1-exp(model.ecp_del*t)) * (1+model.ecp_dec*exp(model.ecp_dec_del/t)) * ( 1 );
 
-        extended_cp_curve_power[i] = QwtIntervalSample(t, (stacked?power_wprime + power_wsecond:0), (stacked?power_wprime + power_wsecond + power_cp:power_cp));
+        extended_cp_curve_power[i] = QwtIntervalSample(t, (stacked?(inversed?0:power_wprime + power_wsecond):0), (stacked?(inversed?power_cp : power_wprime + power_wsecond + power_cp):power_cp));
     }
 
     QwtPlotIntervalCurve *extendedCPCurve = new QwtPlotIntervalCurve("eCP_5_3_CP");
@@ -1351,29 +1359,29 @@ ExtendedCriticalPower::getPlotCurveForExtendedCP_5_3_CP(Model_eCP model, bool st
  * Version 6
  *
  **************************************************/
-Model_eCP
+TestModel
 ExtendedCriticalPower::deriveExtendedCP_6_3_Parameters(bool usebest, RideFileCache *bests, RideFile::SeriesType series, double sanI1, double sanI2, double anI1, double anI2, double aeI1, double aeI2, double laeI1, double laeI2)
 {
-    Model_eCP model;
+    TestModel model;
     model.version = "eCP v6.3";
 
     // bounds on anaerobic interval in minutes
-    const double t1 = sanI1;
-    const double t2 = sanI2;
+    const double t1 = sanI1 / 60.0;
+    const double t2 = sanI2 / 60.0;
 
     // bounds on anaerobic interval in minutes
-    const double t3 = anI1;
-    const double t4 = anI2;
+    const double t3 = anI1 / 60.0;
+    const double t4 = anI2 / 60.0;
 
     // bounds on aerobic interval in minutes
-    const double t5 = aeI1;
-    const double t6 = aeI2;
+    const double t5 = aeI1 / 60.0;
+    const double t6 = aeI2 / 60.0;
 
     // bounds on long aerobic interval in minutes
-    const double t7 = laeI1;
-    const double t8 = laeI2;
+    const double t7 = laeI1 / 60.0;
+    const double t8 = laeI2 / 60.0;
 
-    // bounds of these time valus in the data
+    // bounds of these time values in the data
     int i1, i2, i3, i4, i5, i6, i7, i8;
 
     // find the indexes associated with the bounds
@@ -1595,14 +1603,14 @@ ExtendedCriticalPower::deriveExtendedCP_6_3_Parameters(bool usebest, RideFileCac
     model.pMax = model.paa*(1.10-(1.10-1)*exp(-8*(1/60.0)))*exp(model.paa_dec*(1/60.0)) + model.ecp * (1+model.ecp_dec*exp(model.ecp_dec_del/(1/60.0))) * ( 1 * (1-exp(model.ecp_del*(1/60.0))) + pow((1-exp(model.tau_del*(1/60.0))),2) * model.etau/(1/60.0));
     model.mmp60 = model.paa*(1.10-(1.10-1)*exp(-8*60.0))*exp(model.paa_dec*(60.0)) + model.ecp * (1+model.ecp_dec*exp(model.ecp_dec_del/60.0)) * ( 1 * (1-exp(model.ecp_del*60.0)) + pow((1-exp(model.tau_del*(60.0))),2) * model.etau/(60.0));
 
-    qDebug() <<"eCP(6.3) " << "paa" << model.paa  << "ecp" << model.ecp << "etau" << model.etau << "paa_dec" << model.paa_dec << "ecp_del" << model.ecp_del << "ecp_dec" << model.ecp_dec << "ecp_dec_del" << model.ecp_dec_del;
-    qDebug() <<"eCP(6.3) " << "pmax" << model.pMax << "mmp60" << model.mmp60;
+    qDebug() <<"-> eCP(6.3) " << "paa" << model.paa  << "ecp" << model.ecp << "etau" << model.etau << "paa_dec" << model.paa_dec << "ecp_del" << model.ecp_del << "ecp_dec" << model.ecp_dec << "ecp_dec_del" << model.ecp_dec_del;
+    qDebug() <<"-> eCP(6.3) " << "pmax" << model.pMax << "mmp60" << model.mmp60;
 
     return model;
 }
 
 QwtPlotCurve*
-ExtendedCriticalPower::getPlotCurveForExtendedCP_6_3(Model_eCP model)
+ExtendedCriticalPower::getPlotCurveForExtendedCP_6_3(TestModel model)
 {
     const int extendedCurve2_points = 1000;
 
@@ -1621,7 +1629,7 @@ ExtendedCriticalPower::getPlotCurveForExtendedCP_6_3(Model_eCP model)
     QwtPlotCurve *extendedCPCurve2 = new QwtPlotCurve("eCP_6_3");
     if (appsettings->value(NULL, GC_ANTIALIAS, true).toBool() == true)
         extendedCPCurve2->setRenderHint(QwtPlotItem::RenderAntialiased);
-    QPen e2pen(Qt::cyan);
+    QPen e2pen(Qt::blue);
     e2pen.setWidth(1);
     e2pen.setStyle(Qt::DashLine);
     extendedCPCurve2->setPen(e2pen);
@@ -1631,7 +1639,7 @@ ExtendedCriticalPower::getPlotCurveForExtendedCP_6_3(Model_eCP model)
 }
 
 QwtPlotIntervalCurve*
-ExtendedCriticalPower::getPlotCurveForExtendedCP_6_3_WSecond(Model_eCP model, bool)
+ExtendedCriticalPower::getPlotCurveForExtendedCP_6_3_WSecond(TestModel model, bool)
 {
     const int extendedCurve2_points = 1000;
 
@@ -1671,7 +1679,7 @@ ExtendedCriticalPower::getPlotCurveForExtendedCP_6_3_WSecond(Model_eCP model, bo
 }
 
 QwtPlotIntervalCurve*
-ExtendedCriticalPower::getPlotCurveForExtendedCP_6_3_WPrime(Model_eCP model, bool stacked)
+ExtendedCriticalPower::getPlotCurveForExtendedCP_6_3_WPrime(TestModel model, bool stacked)
 {
     const int extendedCurve2_points = 1000;
 
@@ -1713,7 +1721,7 @@ ExtendedCriticalPower::getPlotCurveForExtendedCP_6_3_WPrime(Model_eCP model, boo
 }
 
 QwtPlotIntervalCurve*
-ExtendedCriticalPower::getPlotCurveForExtendedCP_6_3_CP(Model_eCP model, bool stacked)
+ExtendedCriticalPower::getPlotCurveForExtendedCP_6_3_CP(TestModel model, bool stacked)
 {
     const int extendedCurve2_points = 1000;
 
@@ -1771,10 +1779,10 @@ ExtendedCriticalPower::decimateData(QVector<double> data)
    }
 }*/
 
-Model_eCP
+TestModel
 ExtendedCriticalPower::deriveDanVeloclinicCP_Parameters(bool usebest, RideFileCache *bests, RideFile::SeriesType series, double sanI1, double sanI2, double anI1, double anI2, double aeI1, double aeI2, double laeI1, double laeI2)
 {
-    Model_eCP model;
+    TestModel model;
     model.version = "2 components (Dan v1)";
 
     // bounds on anaerobic interval in minutes
@@ -1793,7 +1801,7 @@ ExtendedCriticalPower::deriveDanVeloclinicCP_Parameters(bool usebest, RideFileCa
     const double t7 = laeI1;
     const double t8 = laeI2;
 
-    // bounds of these time valus in the data
+    // bounds of these time values in the data
     int i1, i2, i3, i4, i5, i6, i7, i8;
 
     // find the indexes associated with the bounds
@@ -2021,3 +2029,164 @@ ExtendedCriticalPower::deriveDanVeloclinicCP_Parameters(bool usebest, RideFileCa
 
     return model;
 }
+
+QwtPlotCurve*
+ExtendedCriticalPower::getPlotCurveFor10secRollingAverage(RideFileCache *bests, RideFile::SeriesType series) {
+    int count = bests->meanMaxArray(series).count();
+    if (count == 0)
+        return NULL;
+
+    QVector<double> smooth;
+    QVector<double> smoothed;
+    QVector<double> time;
+
+    double total = 0.0;
+
+    int l = 6;
+    int d = 3;
+    for (int i = 1; i < 1 + d; i++) {
+        double value = bests->meanMaxArray(series)[i];
+
+        smooth.append(value);
+        total += value;
+    }
+
+    for (int i=1; i < count - d; i++) {
+        double newvalue = bests->meanMaxArray(series)[i+d];
+
+        smooth.append(newvalue);
+        total += newvalue;
+
+        if (smooth.count() > l) {
+            total -= smooth.at(0);
+            smooth.remove(0);
+        }
+
+        smoothed.append(total/smooth.count());
+        time.append(i/60.0f);
+
+    }
+
+
+    QwtPlotCurve *curve = new QwtPlotCurve("rolling");
+    if (appsettings->value(NULL, GC_ANTIALIAS, true).toBool() == true)
+        curve->setRenderHint(QwtPlotItem::RenderAntialiased);
+    QPen e2pen(Qt::green);
+    e2pen.setWidth(1);
+    e2pen.setStyle(Qt::DashLine);
+    curve->setPen(e2pen);
+    curve->setSamples(time.data(), smoothed.data(), smoothed.count());
+
+    return curve;
+}
+
+CpPlotCurve*
+ExtendedCriticalPower::getPlotCurveForQualityPoint(RideFileCache *bests, RideFile::SeriesType series) {
+    int count = bests->meanMaxArray(series).count();
+
+    QVector<double> smooth;
+    //QVector<double> smoothed;
+    QVector<QwtPoint3D> heatSamples;
+
+    double total = 0.0;
+
+    int l = 6;
+    int d = 3;
+    for (int i = 1; i < 1 + d; i++) {
+        double value = bests->meanMaxArray(series)[i];
+
+        smooth.append(value);
+        total += value;
+    }
+
+    for (int i=1; i < count - d; i++) {
+        double newvalue = bests->meanMaxArray(series)[i+d];
+
+        smooth.append(newvalue);
+        total += newvalue;
+
+        if (smooth.count() > l) {
+            total -= smooth.at(0);
+            smooth.remove(0);
+        }
+
+        //smoothed.append(total/smooth.count());
+
+        double value = bests->meanMaxArray(series)[i];
+        double heat = 500*((value-total/smooth.count())/value);
+
+        if ((value-total/smooth.count())/value > 0.0005)
+            heat = 1000;
+
+        QwtPoint3D add(i/60.00f, value, heat);
+        heatSamples << add;
+
+    }
+
+
+    CpPlotCurve *curve = new CpPlotCurve("rolling");
+    if (appsettings->value(NULL, GC_ANTIALIAS, true).toBool() == true)
+        curve->setRenderHint(QwtPlotItem::RenderAntialiased);
+
+    curve->setPenWidth(1);
+    QwtLinearColorMap *colorMap = new QwtLinearColorMap(Qt::yellow, Qt::green);
+    curve->setColorMap(colorMap);
+    curve->setSamples(heatSamples);
+    return curve;
+}
+
+QwtPlotCurve*
+ExtendedCriticalPower::getPlotCurveForDerived(RideFileCache *bests, RideFile::SeriesType series) {
+    int count = bests->meanMaxArray(series).count();
+
+    QVector<double> xValues;
+    QVector<double> yValues;
+    QVector<double> smoothed;
+    QVector<double> time;
+
+    double total = 0.0;
+
+    int l = 10;
+    int d = 5;
+    for (int i = 1; i < 1 + d; i++) {
+        double newvalue = bests->meanMaxArray(series)[i];
+
+        yValues.append(newvalue);
+        xValues.append((i)/60.0f);
+        total += newvalue;
+    }
+
+    for (int i=1; i < count - d; i++) {
+        double newvalue = bests->meanMaxArray(series)[i+d];
+
+        yValues.append(newvalue);
+        xValues.append((i+d)/60.0f);
+        total += newvalue;
+
+        if (yValues.count() > l) {
+            total -= yValues.at(0);
+            xValues.remove(0);
+            yValues.remove(0);
+        }
+
+        // perform linear regression
+        Statistic regress(xValues.data(), yValues.data(), yValues.count());
+
+        smoothed.append(regress.slope()+1200);
+        time.append(i/60.0f);
+        qDebug() << i << regress.slope();
+
+    }
+
+
+    QwtPlotCurve *curve = new QwtPlotCurve("derived");
+    if (appsettings->value(NULL, GC_ANTIALIAS, true).toBool() == true)
+        curve->setRenderHint(QwtPlotItem::RenderAntialiased);
+    QPen e2pen(Qt::green);
+    e2pen.setWidth(1);
+    e2pen.setStyle(Qt::DashLine);
+    curve->setPen(e2pen);
+    curve->setSamples(time.data(), smoothed.data(), smoothed.count());
+
+    return curve;
+}
