@@ -233,7 +233,12 @@ FitlogFileReader::writeRideFile(Context *context, const RideFile *ride, QFile &f
 
     QByteArray xml = doc.toByteArray(4);
     if (!file.open(QIODevice::WriteOnly)) return(false);
-    if (file.write(xml) != xml.size()) return(false);
+    file.resize(0);
+    QTextStream out(&file);
+    out.setCodec("UTF-8");
+    out.setGenerateByteOrderMark(true);
+    out << xml;
+    out.flush();
     file.close();
     return(true);
 }
