@@ -270,8 +270,12 @@ GcFileReader::writeRideFile(Context *,const RideFile *ride, QFile &file) const
     QByteArray xml = doc.toByteArray(4);
     if (!file.open(QIODevice::WriteOnly))
         return false;
-    if (file.write(xml) != xml.size())
-        return false;
+    file.resize(0);
+    QTextStream out(&file);
+    out.setCodec("UTF-8");
+    out.setGenerateByteOrderMark(true);
+    out << xml;
+    out.flush();
     file.close();
     return true;
 }
