@@ -27,6 +27,8 @@
 #include "JsonRideFile.h"
 #include "Context.h"
 #include "DataProcessor.h"
+#include "MainWindow.h"
+#include "TrainDB.h"
 
 #include <QDebug>
 #include <QMessageBox>
@@ -43,7 +45,7 @@ GcUpgrade::upgradeConfirmedByUser(const QDir &home)
     bool folderUpgradeSuccess =  appsettings->cvalue(home.dirName(), GC_UPGRADE_FOLDER_SUCCESS, false).toBool();
 
     //  reset upgrade flag - in case flag is set "true" and subfolder "/activities" and /config do NOT exist
-    //  or - if they exists - do not contain any files, then something is wrong with the upgrade flag -
+    //  or - if they exists, but config does not contain any files, then something is wrong with the upgrade flag -
     //  (potentially due to a not fitting data restore)
     //  so let's reset the flag to "false" and run the upgrade again - this can never go wrong (if their is
     //  nothing to be upgrade - upgrade a success anyway
@@ -67,9 +69,9 @@ GcUpgrade::upgradeConfirmedByUser(const QDir &home)
     return true; // if there is no upgrade needed, just proceed
 }
 
-int 
+int
 GcUpgrade::upgrade(const QDir &home)
-{ 
+{
 
     // what was the last version? -- do we need to upgrade?
     int last = appsettings->cvalue(home.dirName(), GC_VERSION_USED, 0).toInt();
@@ -114,32 +116,31 @@ GcUpgrade::upgrade(const QDir &home)
             if (weight_ <= 0.00) appsettings->setCValue(home.dirName(), GC_WEIGHT, "75.0");
 
             // 5. startup with common sidebars shown (less ugly)
-            appsettings->setCValue(home.dirName(), "splitter/LTM/hide", true);
-            appsettings->setCValue(home.dirName(), "splitter/LTM/hide/0", false);
-            appsettings->setCValue(home.dirName(), "splitter/LTM/hide/1", false);
-            appsettings->setCValue(home.dirName(), "splitter/LTM/hide/2", false);
-            appsettings->setCValue(home.dirName(), "splitter/LTM/hide/3", true);
-            appsettings->setCValue(home.dirName(), "splitter/analysis/hide", true);
-            appsettings->setCValue(home.dirName(), "splitter/analysis/hide/0", false);
-            appsettings->setCValue(home.dirName(), "splitter/analysis/hide/1", true);
-            appsettings->setCValue(home.dirName(), "splitter/analysis/hide/2", false);
-            appsettings->setCValue(home.dirName(), "splitter/analysis/hide/3", true);
-            appsettings->setCValue(home.dirName(), "splitter/diary/hide", true);
-            appsettings->setCValue(home.dirName(), "splitter/diary/hide/0", false);
-            appsettings->setCValue(home.dirName(), "splitter/diary/hide/1", false);
-            appsettings->setCValue(home.dirName(), "splitter/diary/hide/2", true);
-            appsettings->setCValue(home.dirName(), "splitter/train/hide", true);
-            appsettings->setCValue(home.dirName(), "splitter/train/hide/0", false);
-            appsettings->setCValue(home.dirName(), "splitter/train/hide/1", false);
-            appsettings->setCValue(home.dirName(), "splitter/train/hide/2", false);
-            appsettings->setCValue(home.dirName(), "splitter/train/hide/3", false);
+            appsettings->setCValue(home.dirName(), GC_QSETTINGS_ATHLETE_LAYOUT+QString("splitter/LTM/hide"), true);
+            appsettings->setCValue(home.dirName(), GC_QSETTINGS_ATHLETE_LAYOUT+QString("splitter/LTM/hide/0"), false);
+            appsettings->setCValue(home.dirName(), GC_QSETTINGS_ATHLETE_LAYOUT+QString("splitter/LTM/hide/1"), false);
+            appsettings->setCValue(home.dirName(), GC_QSETTINGS_ATHLETE_LAYOUT+QString("splitter/LTM/hide/2"), false);
+            appsettings->setCValue(home.dirName(), GC_QSETTINGS_ATHLETE_LAYOUT+QString("splitter/LTM/hide/3"), true);
+            appsettings->setCValue(home.dirName(), GC_QSETTINGS_ATHLETE_LAYOUT+QString("splitter/analysis/hide"), true);
+            appsettings->setCValue(home.dirName(), GC_QSETTINGS_ATHLETE_LAYOUT+QString("splitter/analysis/hide/0"), false);
+            appsettings->setCValue(home.dirName(), GC_QSETTINGS_ATHLETE_LAYOUT+QString("splitter/analysis/hide/1"), true);
+            appsettings->setCValue(home.dirName(), GC_QSETTINGS_ATHLETE_LAYOUT+QString("splitter/analysis/hide/2"), false);
+            appsettings->setCValue(home.dirName(), GC_QSETTINGS_ATHLETE_LAYOUT+QString("splitter/analysis/hide/3"), true);
+            appsettings->setCValue(home.dirName(), GC_QSETTINGS_ATHLETE_LAYOUT+QString("splitter/diary/hide"), true);
+            appsettings->setCValue(home.dirName(), GC_QSETTINGS_ATHLETE_LAYOUT+QString("splitter/diary/hide/0"), false);
+            appsettings->setCValue(home.dirName(), GC_QSETTINGS_ATHLETE_LAYOUT+QString("splitter/diary/hide/1"), false);
+            appsettings->setCValue(home.dirName(), GC_QSETTINGS_ATHLETE_LAYOUT+QString("splitter/diary/hide/2"), true);
+            appsettings->setCValue(home.dirName(), GC_QSETTINGS_ATHLETE_LAYOUT+QString("splitter/train/hide"), true);
+            appsettings->setCValue(home.dirName(), GC_QSETTINGS_ATHLETE_LAYOUT+QString("splitter/train/hide/0"), false);
+            appsettings->setCValue(home.dirName(), GC_QSETTINGS_ATHLETE_LAYOUT+QString("splitter/train/hide/1"), false);
+            appsettings->setCValue(home.dirName(), GC_QSETTINGS_ATHLETE_LAYOUT+QString("splitter/train/hide/2"), false);
+            appsettings->setCValue(home.dirName(), GC_QSETTINGS_ATHLETE_LAYOUT+QString("splitter/train/hide/3"), false);
 
             // 6. Delete any old measures.xml -- its for withings only
             QFile msxml(QString("%1/measures.xml").arg(home.canonicalPath()));
             if (msxml.exists()) msxml.remove();
 
-            // FINALLY -- Set latest version - so only tries to upgrade once
-            appsettings->setCValue(home.dirName(), GC_VERSION_USED, VERSION_LATEST);
+
         }
     }
 
@@ -160,6 +161,7 @@ GcUpgrade::upgrade(const QDir &home)
         // 3. Remove metricDBv3 - force rebuild including the search index
         QFile db(QString("%1/metricDBv3").arg(home.canonicalPath()));
         if (db.exists()) db.remove();
+
     }
 
     //----------------------------------------------------------------------
@@ -206,7 +208,7 @@ GcUpgrade::upgrade(const QDir &home)
         QString theme = "Flat";
         QColor chromeColor = QColor(0xec,0xec,0xec);
 #ifdef Q_OS_MAC
-        // Yosemite or earlier 
+        // Yosemite or earlier
         if (QSysInfo::MacintoshVersion >= 12) {
 
             chromeColor = QColor(235,235,235);
@@ -271,15 +273,15 @@ GcUpgrade::upgrade(const QDir &home)
                 if (pos < 0) pos = 1;
 
                 // add them
-                if (indexAnTISS < 0) { 
+                if (indexAnTISS < 0) {
                     add.name = tr("Anaerobic TISS");
                     fieldDefinitions.insert(pos, add);
                 }
-                if (indexAeTISS < 0) { 
+                if (indexAeTISS < 0) {
                     add.name = tr("Aerobic TISS");
                     fieldDefinitions.insert(pos, add);
                 }
-                if (indexTSS < 0) { 
+                if (indexTSS < 0) {
                     add.name = tr("TSS");
                     fieldDefinitions.insert(pos, add);
                 }
@@ -347,6 +349,30 @@ GcUpgrade::upgrade(const QDir &home)
 
     }
 
+
+    //----------------------------------------------------------------------
+    // 3.3 upgrade processing
+    //----------------------------------------------------------------------
+
+    if (last < VERSION33_BUILD) {
+
+        trainDB->upgradeDefaultEntriesWorkout();
+    }
+
+
+    //----------------------------------------------------------------------
+    // ... here any further Release Number dependent Upgrade Process is to be added ...
+    //----------------------------------------------------------------------
+
+
+
+    //----------------------------------------------------------------------
+    // All Version dependent Upgrade Steps are done ...
+    //----------------------------------------------------------------------
+
+    // FINALLY -- Set latest version - so only tries to upgrade once
+    appsettings->setCValue(home.dirName(), GC_VERSION_USED, VERSION_LATEST);
+
     //----------------------------------------------------------------------
     // 3.2 new subfolder introduction and upgrade processing
     //----------------------------------------------------------------------
@@ -369,6 +395,8 @@ GcUpgrade::upgrade(const QDir &home)
 
         AthleteDirectoryStructure newHome(home);
         newHome.createAllSubdirs();
+        //now we can created the QSettings for the Athlete and Migrate the oldseetings
+        appsettings->initializeQSettingsAthlete(gcroot, home.dirName());
 
         if (!newHome.subDirsExist()) {
            upgradeLog->append(QString(tr("Error: Creation of subdirectories failed")),2);
@@ -740,7 +768,7 @@ class FileUtil
 
             bool result = true;
             QDir dir(dirName);
- 
+
             if (dir.exists(dirName)) {
                 foreach(QFileInfo info, dir.entryInfoList(QDir::NoDotAndDotDot | QDir::System | QDir::Hidden  | QDir::AllDirs | QDir::Files, QDir::DirsFirst)) {
                     if (info.isDir()) {
@@ -750,13 +778,13 @@ class FileUtil
 
                         result = QFile::remove(info.absoluteFilePath());
                     }
- 
+
                     if (!result) { return result; }
 
                 }
                 result = dir.rmdir(dirName);
             }
- 
+
         return result;
     }
 
