@@ -19,6 +19,9 @@
 #include "LTMPopup.h"
 #include "MainWindow.h"
 #include "Athlete.h"
+#include "Specification.h"
+#include "RideCache.h"
+#include "RideFileCache.h" // for RideBest
 
 LTMPopup::LTMPopup(Context *context) : QWidget(context->mainWindow), context(context)
 {
@@ -42,7 +45,7 @@ LTMPopup::LTMPopup(Context *context) : QWidget(context->mainWindow), context(con
     QFont titleFont;
     titleFont.setPointSize(14);
     titleFont.setBold(true);
-    title = new QLabel(tr("No Ride Selected"));
+    title = new QLabel(tr("No activity Selected"));
     title->setFont(titleFont);
     title->setFixedHeight(30);
     mainLayout->addWidget(title);
@@ -108,7 +111,7 @@ LTMPopup::setTitle(QString s)
 }
 
 void
-LTMPopup::setData(QList<SummaryMetrics>data, const RideMetric *metric, QString title)
+LTMPopup::setData(Specification spec, const RideMetric *metric, QString title)
 {
     // create the ride list
     int count = 0;
@@ -129,15 +132,15 @@ LTMPopup::setData(QList<SummaryMetrics>data, const RideMetric *metric, QString t
     rides->setHorizontalHeaderItem(1,h);
 
     // now add rows to the table for each entry
-    foreach(SummaryMetrics x, data) {
+    foreach(RideItem *item, context->athlete->rideCache->rides()) {
 
-        QDateTime rideDate = x.getRideDate();
+        if (!spec.pass(item)) continue;
 
-        // we'll select it for summary aggregation
-        selected << x;
+        // what rides were selected ?
+        selected << item->fileName;
 
         // date/time
-        QTableWidgetItem *t = new QTableWidgetItem(rideDate.toString(tr("ddd, dd MMM yy hh:mmA")));
+        QTableWidgetItem *t = new QTableWidgetItem(item->dateTime.toString(tr("ddd, dd MMM yy hh:mm")));
         t->setFlags(t->flags() & (~Qt::ItemIsEditable));
         t->setTextAlignment(Qt::AlignHCenter);
         rides->setRowCount(count+1);
@@ -145,13 +148,15 @@ LTMPopup::setData(QList<SummaryMetrics>data, const RideMetric *metric, QString t
         rides->setRowHeight(count, 14);
 
         // metrics
-        QString value = x.getStringForSymbol(metric->symbol(), context->athlete->useMetricUnits);
+        double d = item->getForSymbol(metric->symbol());
+        const_cast<RideMetric *>(metric)->setValue(d);
+        QString value = metric->toString(context->athlete->useMetricUnits);
+
         h = new QTableWidgetItem(value,QTableWidgetItem::Type);
         h->setFlags(t->flags() & (~Qt::ItemIsEditable));
         h->setTextAlignment(Qt::AlignHCenter);
         rides->setItem(count, 1, h);
 
-
         count++;
     }
 
@@ -170,7 +175,7 @@ LTMPopup::setData(QList<SummaryMetrics>data, const RideMetric *metric, QString t
        rides->hide();
        metrics->hide();
        notes->hide();
-       title = tr("No ride selected");
+       title = tr("No activity selected");
     } else {
        // one or more rides
        rides->show();
@@ -264,22 +269,23 @@ LTMPopup::setData(LTMSettings &settings, QDate start, QDate end, QTime time)
 
     // Summary Metrics.data is always available and thus perfect find the rides eligible for the list
     int i = 0;
-    foreach(SummaryMetrics data, (*settings.data)) {
-        QDateTime rideDate = data.getRideDate();
+    foreach(RideItem *item, context->athlete->rideCache->rides()) {
+
+        if (!settings.specification.pass(item)) continue;
+
+        QDate rideDate = item->dateTime.date();
 
           // check either RideDate (for all Date related groupBy's) or RideTime (for LTM_TOD only)
-          if (((settings.groupBy != LTM_TOD) && (rideDate.date() >= start && rideDate.date() <= end))
-            ||((settings.groupBy == LTM_TOD) && (rideDate.time() >= time && rideDate.time() <= end_time))) {
-
-              // apply filters
-             if (context->isfiltered && !context->filters.contains(data.getFileName())) continue;
-             if (context->ishomefiltered && !context->homeFilters.contains(data.getFileName())) continue;
+          if (((settings.groupBy != LTM_TOD) && (rideDate >= start && rideDate <= end))
+            ||
+            // group by time of day ?
+            ((settings.groupBy == LTM_TOD) && (item->dateTime.time() >= time && item->dateTime.time() <= end_time))) {
 
              // we'll select it for summary display
-             selected << data;
+             selected << item->fileName;
 
              // date/time
-             QTableWidgetItem *t = new QTableWidgetItem(rideDate.toString(tr("ddd, dd MMM yy hh:mmA")));
+             QTableWidgetItem *t = new QTableWidgetItem(item->dateTime.toString(tr("ddd, dd MMM yy hh:mm")));
              t->setFlags(t->flags() & (~Qt::ItemIsEditable));
              t->setTextAlignment(Qt::AlignHCenter);
              rides->setRowCount(count+1);
@@ -292,10 +298,15 @@ LTMPopup::setData(LTMSettings &settings, QDate start, QDate end, QTime time)
              foreach(MetricDetail d, settings.metrics) {
                  QString value;
                  if (d.type == METRIC_DB || d.type == METRIC_META) {
-                     value = data.getStringForSymbol(d.symbol, context->athlete->useMetricUnits);
+
+                     // get the value to show as a string
+                     value = item->getStringForSymbol(d.symbol, context->athlete->useMetricUnits);
+
                  } else if (d.type == METRIC_BEST) {
+
                      // bests are only available if a METRIC_BEST exists
-                     SummaryMetrics bests = settings.bests->at(i);
+                     RideBest bests = settings.bests->at(i);
+
                      // and are not considered as standard metrics for rounding, so do it here with precision 0
                      double v = bests.getForSymbol(d.bestSymbol);
                      value = QString("%1").arg(v, 0, 'f', 0);
@@ -329,43 +340,43 @@ LTMPopup::setData(LTMSettings &settings, QDate start, QDate end, QTime time)
        rides->hide();
        metrics->hide();
        notes->hide();
-       _title = tr("No ride selected");
+       _title = tr("No activity selected");
     } else {
        // one or more rides
        rides->show();
        metrics->show();
        notes->show();
        // give some extrat info
-       if (count > 1) _title += QString(tr(" (%1 rides)")).arg(count);
+       if (count > 1) _title += QString(tr(" (%1 activities)")).arg(count);
     }
 
-    if (nonRideMetrics) _title += QString(tr(" / non ride-related metrics skipped"));
+    if (nonRideMetrics) _title += QString(tr(" / non activity-related metrics skipped"));
 
     setTitle(_title);
-
     rideSelected();
 }
 
 QString
-LTMPopup::setSummaryHTML(SummaryMetrics &results)
+LTMPopup::setSummaryHTML(RideItem *item)
 {
     // where we construct the text
     QString summaryText("");
 
     // main totals
-    static const QStringList totalColumn = QStringList()
+    const QStringList totalColumn = QStringList()
         << "workout_time"
         << "time_riding"
-        << "total_distance"
+        << (item->isSwim ? "distance_swim" : "total_distance")
         << "total_work"
         << "skiba_wprime_exp"
         << "elevation_gain";
 
-    static const QStringList averageColumn = QStringList()
+    const QStringList averageColumn = QStringList()
         << "average_speed"
         << "average_power"
         << "average_hr"
-        << "average_cad";
+        << "average_cad"
+        << (item->isSwim ? "pace_swim" : "pace");
 
     static const QStringList maximumColumn = QStringList()
         << "max_speed"
@@ -435,12 +446,13 @@ LTMPopup::setSummaryHTML(SummaryMetrics &results)
 
         const RideMetric *metric = RideMetricFactory::instance().rideMetric(metricname);
 
-        QStringList empty; // filter list not used at present
-        QList<SummaryMetrics> resultList1;
-        resultList1 << results;
-
-        // use getAggregated even if it's only 1 file to have consistent value treatment
-        QString value = SummaryMetrics::getAggregated(context, metricname, resultList1, empty, false, context->athlete->useMetricUnits);
+        // use getAggregate even if it's only 1 file to have consistent value treatment
+        double d = item->getForSymbol(metricname, true);
+        QString value;
+        if (metric) {
+            const_cast<RideMetric*>(metric)->setValue(d);
+            value = metric->toString(context->athlete->useMetricUnits);
+        }
 
         // Maximum Max and Average Average looks nasty, remove from name for display
         QString s = metric ? metric->name().replace(QRegExp(tr("^(Average|Max) ")), "") : "unknown";
@@ -475,21 +487,30 @@ LTMPopup::setSummaryHTML(SummaryMetrics &results)
 
 
    return summaryText;
-
 }
 
 void
 LTMPopup::rideSelected()
 {
+    // which ride is selected
     int index = 0;
-    foreach (QTableWidgetItem *item, rides->selectedItems())
-        index = item->row();
+    if (rides->selectedItems().count())
+        index = rides->selectedItems().first()->row();
 
+    // do we have any rides and is the index within bounds
     if (selected.count() > index) {
 
-        // update summary
-        metrics->setText(setSummaryHTML(selected[index]));
-        notes->setText(selected[index].getText("Notes", ""));
+        RideItem *have = context->athlete->rideCache->getRide(selected[index]);
+
+        if (have) {
+
+            // update summary
+            metrics->setText(""); //! stop crash (?)
+            metrics->setText(setSummaryHTML(have));
+
+            notes->setText(""); //! stop crash (?)
+            notes->setText(have->getText("Notes", ""));
+        }
     }
     resizeEvent(NULL);
 }
