@@ -94,6 +94,10 @@ class MainWindow : public QMainWindow
         void support();
         void actionClicked(int);
 
+        // search box gets and loses focus - make big/small
+        void searchFocusIn();
+        void searchFocusOut();
+
         // open and closing windows and tabs
         void closeAll();    // close all windows and tabs
 
@@ -112,6 +116,10 @@ class MainWindow : public QMainWindow
 
         void switchTab(int index); // for switching between one tab and another
 
+        // Athlete Backup
+        void setBackupAthleteMenu();
+        void backupAthlete(QString name);
+
         // Search / Filter
         void setFilter(QStringList);
         void clearFilter();
@@ -143,6 +151,7 @@ class MainWindow : public QMainWindow
         // Analysis View
         void showTools();
         void showRhoEstimator();
+        void showVDOTCalculator();
 
         // Training View
         void addDevice();
@@ -179,6 +188,12 @@ class MainWindow : public QMainWindow
         void uploadTP();
         void downloadTP();
 #endif
+#if QT_VERSION > 0x050000
+        void uploadDropbox();
+        void syncDropbox();
+#endif
+        void uploadLocalFileStore();
+        void syncLocalFileStore();
         void importFile();
         void splitRide();
         void mergeRide();
@@ -231,6 +246,10 @@ class MainWindow : public QMainWindow
         QMenu *openWindowMenu, *openTabMenu;
         QSignalMapper *windowMapper, *tabMapper;
 
+        // backup
+        QMenu *backupAthleteMenu;
+        QSignalMapper *backupMapper;
+
         // chart menus
         QMenu *chartMenu;
         QMenu *subChartMenu;
