@@ -377,14 +377,6 @@ ManualRideDialog::~ManualRideDialog() {
 void
 ManualRideDialog::estimate()
 {
-    QTime time = duration->time();
-    double hours = (time.hour()) + (time.minute() / 60.00) + (time.second() / 3600.00);
-    double dist = distance->value();
-
-    // Estimate average speed
-    if (hours && dist) avgKPH->setValue(dist / hours);
-    else avgKPH->setValue(0.0);
-
     if (byManual->isChecked()) {
         BS->setEnabled(true);
         DP->setEnabled(true);
@@ -404,6 +396,9 @@ ManualRideDialog::estimate()
 
     if (byDuration->isChecked()) {
         // by time
+        QTime time = duration->time();
+        double hours = (time.hour()) + (time.minute() / 60.00) + (time.second() / 3600.00);
+
         BS->setValue(hours * timeBS);
         DP->setValue(hours * timeDP);
         TSS->setValue(hours * timeTSS);
@@ -411,6 +406,8 @@ ManualRideDialog::estimate()
 
     } else {
         // by distance
+        double dist = distance->value();
+
         BS->setValue(dist * distanceBS);
         DP->setValue(dist * distanceDP);
         TSS->setValue(dist * distanceTSS);
