@@ -22,13 +22,13 @@
 #include <QTextStream>
 #include <algorithm> // for std::sort
 #include "cmath"
-
+#include <assert.h>
 
 static int polarFileReaderRegistered =
     RideFileFactory::instance().registerReader(
         "hrm", "Polar Precision", new PolarFileReader());
 
-RideFile *PolarFileReader::openRideFile(QFile &file, QStringList &errors, QList<RideFile*>*) const
+RideFile *PolarFileReader::openRideFile(QFile &file, QStringList &errors, QList<RideFile*>*rideList) const
 {
 /*
 * Polar HRM file format documented at www.polar.fi/files/Polar_HRM_file%20format.pdf
@@ -46,16 +46,35 @@ RideFile *PolarFileReader::openRideFile(QFile &file, QStringList &errors, QList<
     double seconds=0;
     double distance=0;
     int interval = 0;
+    int StartDelay = 0;
 
     bool speed = false;
     bool cadence = false;
     bool altitude = false;
     bool power = false;
     bool balance = false;
-
-
+    bool haveGPX = false;
+    int igpx = 0;
+    int ngpx = 0;
+    double lat=0,lon=0;
     int recInterval = 1;
 
+    // Read Polar GPX file (if exist with same name as hrm file).
+    RideFile *gpxresult;
+    RideFilePoint *p;
+    QString suffix = file.fileName();
+    int dot = suffix.lastIndexOf(".");
+    assert(dot >= 0);
+    QFile gpxfile(suffix.left(dot)+".gpx");
+    haveGPX = gpxfile.exists();
+    
+    if (haveGPX)
+      {
+	GpxFileReader reader;
+	gpxresult = reader.openRideFile(gpxfile,errors,rideList);
+	ngpx = gpxresult->dataPoints().count();
+      }
+      
     if (!file.open(QFile::ReadOnly)) {
         errors << ("Could not open ride file: \""
                    + file.fileName() + "\"");
@@ -165,7 +184,13 @@ this differently
 
                 } else if (line.contains("Interval=")) {
                     recInterval = line.remove(0,9).toInt();
-                    rideFile->setRecIntSecs(recInterval);
+		    if (recInterval==238){
+		        /* This R-R data */
+		        rideFile->setRecIntSecs(1);
+		    }
+		    else {
+		      rideFile->setRecIntSecs(recInterval);
+		    }
                 } else if (line.contains("Date=")) {
                     line.remove(0,5);
                     date= QDate(line.left(4).toInt(),
@@ -178,8 +203,15 @@ this differently
                                             line.mid(3,2).toInt(),
                                             line.mid(6,2).toInt()));
                     rideFile->setStartTime(datetime);
-                 }
-
+		} else if (line.contains("StartDelay=")){
+		  StartDelay = line.remove(0,11).toInt();
+		  if (recInterval==238){
+		    seconds = StartDelay/1000.0;
+		  }
+		  else{
+		    seconds = recInterval;
+		  }
+		}
 
             }
             else if (section == "[Note]"){
@@ -203,17 +235,17 @@ this differently
                 }
             }
             else if (section == "[HRData]"){
-                double nm=0,kph=0,watts=0,km=0,cad=0,hr=0,alt=0;
+	        double nm=0,kph=0,watts=0,km=0,cad=0,hr=0,alt=0,hrm=0;
                 double lrbalance=0;
 
-                seconds += recInterval;
-
-                int i=0;
-                hr = line.section('\t', i, i).toDouble();
+		int i=0;
+		hrm = line.section('\t', i, i).toDouble();
                 i++;
 
                 if (speed) {
                     kph = line.section('\t', i, i).toDouble()/10;
+		    distance += kph/60/60*recInterval;
+		    km = distance;
                     i++;
                 }
                 if (cadence) {
@@ -245,9 +277,6 @@ this differently
                     i++;
                 }
 
-                distance = distance + kph/60/60*recInterval;
-                km = distance;
-
                 if (next_interval < seconds) {
                     interval = intervals.indexOf(next_interval);
                     if (intervals.count()>interval+1){
@@ -262,14 +291,47 @@ this differently
                     alt *= METERS_PER_FOOT;
                 }
 
-                rideFile->appendPoint(seconds, cad, hr, km, kph, nm, watts, alt, 0.0, 0.0, 0.0, 0.0, RideFile::NoTemp, lrbalance, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, interval);
-	            //fprintf(stderr, " %f, %f, %f, %f, %f, %f, %f, %d\n", seconds, cad, hr, km, kph, nm, watts, alt, interval);
+		if (recInterval==238){
+		  hr = 60000.0/hrm;
+		} else {
+		  hr = hrm;
+		}
+		
+		if (haveGPX && (igpx<ngpx))
+		  {
+		    p = gpxresult->dataPoints()[igpx];
+		    // Use previous value if GPS is momentarely
+		    // lost. Should have option for interpolating.
+		    if (p->lat!=0.0 && p->lon!=0.0){
+		      lat = p->lat;
+		      lon = p->lon;
+		      // Must check if current HRM speed is zero while
+		      // we have GPX speed
+		      if (kph==0.0 && p->kph>1.0)
+			{
+			  kph = p->kph;
+			  distance += kph/60/60*recInterval;
+			  km = distance;
+			}
+		    }
+		    if (seconds>=p->secs)
+		      igpx += 1;
+		  }
+		
+		rideFile->appendPoint(seconds, cad, hr, km, kph, nm, watts, alt, lon, lat, 0.0, 0.0, RideFile::NoTemp, lrbalance, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, interval);
+		// fprintf(stderr, " %f, %f, %f, %f, %f, %f, %f, %d\n", seconds, cad, hr, km, kph, nm, watts, alt, interval);
+		if (recInterval==238){
+		  seconds += hrm / 1000.0;
+		} else {
+		  seconds += recInterval;
+		}
             }
 
         ++lineno;
         }
     }
 
+    rideFile->setTag("Notes", note);
     QRegExp rideTime("^.*/(\\d\\d\\d\\d)_(\\d\\d)_(\\d\\d)_"
                      "(\\d\\d)_(\\d\\d)_(\\d\\d)\\.hrm$");
     if (rideTime.indexIn(file.fileName()) >= 0) {
