@@ -26,6 +26,10 @@
 #include "Athlete.h"
 #include "Zones.h"
 #include "HrZones.h"
+#include "PaceZones.h"
+#include "Specification.h"
+#include "Settings.h"
+#include "Colors.h"
 
 #include <qwt_plot.h>
 #include <qwt_plot_canvas.h>
@@ -42,6 +46,7 @@ class QwtPlotCurve;
 class QwtPlotGrid;
 class Context;
 class RideItem;
+class IntervalItem;
 struct RideFilePoint;
 class RideFileCache;
 class HistogramWindow;
@@ -49,9 +54,10 @@ class PowerHistBackground;
 class PowerHistZoneLabel;
 class HrHistBackground;
 class HrHistZoneLabel;
+class PaceHistBackground;
+class PaceHistZoneLabel;
 class LTMCanvasPicker;
 class ZoneScaleDraw;
-class SummaryMetrics;
 
 class penTooltip: public QwtPlotZoomer
 {
@@ -63,20 +69,24 @@ class penTooltip: public QwtPlotZoomer
              setTrackerMode(AlwaysOn);
          }
 
+        virtual QRect trackerRect(const QFont &font) const
+        {
+            return QwtPlotPicker::trackerRect(font).adjusted(-3,-3,3,3);
+        }
+
         virtual QwtText trackerText(const QPoint &/*pos*/) const {
-            QColor bg = QColor(Qt::lightGray);
-#if QT_VERSION >= 0x040300
-            bg.setAlpha(200);
-#endif
-            QwtText text;
-            QFont def;
-            //def.setPointSize(8); // too small on low res displays (Mac)
-            //double val = ceil(pos.y()*100) / 100; // round to 2 decimal place
-            //text.setText(QString("%1 %2").arg(val).arg(format), QwtText::PlainText);
-            text.setText(tip);
-            text.setFont(def);
-            text.setBackgroundBrush( QBrush( bg ));
-            text.setRenderFlags(Qt::AlignLeft | Qt::AlignTop);
+            QwtText text(tip);
+
+            QFont stGiles;
+            stGiles.fromString(appsettings->value(this, GC_FONT_CHARTLABELS, QFont().toString()).toString());
+            stGiles.setPointSize(appsettings->value(NULL, GC_FONT_CHARTLABELS_SIZE, 8).toInt());
+            stGiles.setWeight(QFont::Bold);
+            text.setFont(stGiles);
+
+            text.setBackgroundBrush(QBrush( GColor(CPLOTMARKER)));
+            text.setColor(GColor(CRIDEPLOTBACKGROUND));
+            text.setBorderRadius(6);
+            text.setRenderFlags(Qt::AlignCenter | Qt::AlignVCenter);
             return text;
         }
 
@@ -93,18 +103,20 @@ class HistData // each curve needs a lot of data (!? this may need refactoring,
     public:
 
         // storage for data counts
-        QVector<unsigned int> aPowerArray, wattsArray, wattsZoneArray, wattsCPZoneArray, wattsKgArray, nmArray, hrArray,
-                              hrZoneArray, kphArray, cadArray, metricArray;
+        QVector<unsigned int> aPowerArray, wattsArray, wbalArray, wbalZoneArray,
+                              wattsZoneArray, wattsCPZoneArray, wattsKgArray, nmArray,
+                              hrArray, hrZoneArray, hrCPZoneArray,
+                              kphArray, paceZoneArray, paceCPZoneArray,
+                              cadArray, gearArray, smo2Array, metricArray;
 
         // storage for data counts in interval selected
-        QVector<unsigned int> aPowerSelectedArray, 
-                              wattsSelectedArray, 
-                              wattsZoneSelectedArray, 
-                              wattsCPZoneSelectedArray,
-                              wattsKgSelectedArray,
-                              nmSelectedArray, hrSelectedArray,
-                              hrZoneSelectedArray, kphSelectedArray,
-                              cadSelectedArray;
+        QVector<unsigned int> aPowerSelectedArray, wattsSelectedArray, 
+                              wattsZoneSelectedArray, wattsCPZoneSelectedArray,
+                              wbalSelectedArray, wbalZoneSelectedArray,
+                              wattsKgSelectedArray, nmSelectedArray,
+                              hrSelectedArray, hrZoneSelectedArray, hrCPZoneSelectedArray,
+                              kphSelectedArray, paceZoneSelectedArray, paceCPZoneSelectedArray,
+                              cadSelectedArray, smo2SelectedArray, gearSelectedArray;
 };
 
 class PowerHist : public QwtPlot
@@ -112,6 +124,8 @@ class PowerHist : public QwtPlot
     Q_OBJECT
     G_OBJECT
 
+    friend class ::PaceHistBackground;
+    friend class ::PaceHistZoneLabel;
     friend class ::HrHistBackground;
     friend class ::HrHistZoneLabel;
     friend class ::PowerHistBackground;
@@ -137,7 +151,7 @@ class PowerHist : public QwtPlot
         void setData(RideItem *_rideItem, bool force=false);
 
         // used to set and bin ride data
-        void setArraysFromRide(RideFile *ride, HistData &standard, const Zones *zones, RideFileInterval hover);
+        void setArraysFromRide(RideFile *ride, HistData &standard, const Zones *zones, IntervalItem *hover);
         void binData(HistData &standard, QVector<double>&, QVector<double>&, QVector<double>&, QVector<double>&);
 
         // set data from the compare intervals -or- dateranges
@@ -148,15 +162,14 @@ class PowerHist : public QwtPlot
         void setData(RideFileCache *source);
 
         // set data from metrics
-        void setData(QList<SummaryMetrics>&results, QString totalMetric, QString distMetric,
-                     bool isFiltered, QStringList files, HistData *data);
+        void setData(Specification spec, QString totalMetric, QString distMetric, HistData *data);
 
         void setlnY(bool value);
         void setWithZeros(bool value);
         void setZoned(bool value);
         void setCPZoned(bool value);
         void setSumY(bool value);
-        void configChanged();
+        void configChanged(qint32);
         void setAxisTitle(int axis, QString label);
         void setYMax();
         void setBinWidth(double value);
@@ -171,7 +184,7 @@ class PowerHist : public QwtPlot
 
         // react to plot signals
         void pointHover(QwtPlotCurve *curve, int index);
-        void intervalHover(RideFileInterval);
+        void intervalHover(IntervalItem*);
 
         // get told to refresh
         void recalc(bool force=false); // normal mode recalc
@@ -187,13 +200,17 @@ class PowerHist : public QwtPlot
 
     protected:
 
+        bool isZoningEnabled();
         void refreshHRZoneLabels();
+        void refreshPaceZoneLabels();
         void setParameterAxisTitle();
         bool isSelected(const RideFilePoint *p, double);
+        bool isSelected(const double t, double sample);
         void percentify(QVector<double> &, double factor); // and a function to convert
 
         bool shadeZones() const; // check if zone shading is both wanted and possible
         bool shadeHRZones() const; // check if zone shading is both wanted and possible
+        bool shadePaceZones() const; // check if zone shading is both wanted and possible
 
         // plot settings
         RideItem *rideItem;
@@ -216,6 +233,7 @@ class PowerHist : public QwtPlot
         QwtPlotGrid *grid;
         PowerHistBackground *bg;
         HrHistBackground *hrbg;
+        PaceHistBackground *pacebg;
         penTooltip *zoomer;
         LTMCanvasPicker *canvasPicker;
 
@@ -228,6 +246,7 @@ class PowerHist : public QwtPlot
         // background shading
         QList <PowerHistZoneLabel *> zoneLabels;
         QList <HrHistZoneLabel *> hrzoneLabels;
+        QList <PaceHistZoneLabel *> pacezoneLabels;
 
         // zone data labels (actual values)
         QList <QwtPlotMarker *> zoneDataLabels;
@@ -294,8 +313,9 @@ public:
 	if (! rideItem)
 	    return;
 
-	const Zones *zones       = rideItem->zones;
-	int zone_range     = rideItem->zoneRange();
+	const Zones *zones = parent->context->athlete->zones();
+        int zone_range = -1;
+        if (zones) zone_range = zones->whichRange(rideItem->dateTime.date());
 
 	if (parent->shadeZones() && (zone_range >= 0)) {
 	    QList <int> zone_lows = zones->getZoneLows(zone_range);
@@ -356,8 +376,9 @@ public:
 	if (! rideItem)
 	    return;
 
-	const Zones *zones       = rideItem->zones;
-	int zone_range     = rideItem->zoneRange();
+	const Zones *zones = parent->context->athlete->zones();
+        int zone_range = -1;
+        if (zones) zone_range = zones->whichRange(rideItem->dateTime.date());
 
 	setZ(1.0 + zone_number / 100.0);
 
@@ -446,8 +467,9 @@ public:
 	if (! rideItem)
 	    return;
 
-	const HrZones *zones       = parent->context->athlete->hrZones();
-	int zone_range     = rideItem->hrZoneRange();
+	const HrZones *zones = parent->context->athlete->hrZones();
+        int zone_range = -1;
+        if (zones) zone_range = zones->whichRange(rideItem->dateTime.date());
 
 	if (parent->shadeHRZones() && (zone_range >= 0)) {
 	    QList <int> zone_lows = zones->getZoneLows(zone_range);
@@ -496,8 +518,9 @@ public:
 	if (! rideItem)
 	    return;
 
-	const HrZones *zones       = parent->context->athlete->hrZones();
-	int zone_range     = rideItem->hrZoneRange();
+	const HrZones *zones = parent->context->athlete->hrZones();
+        int zone_range = -1;
+        if (zones) zone_range = zones->whichRange(rideItem->dateTime.date());
 
 	setZ(1.0 + zone_number / 100.0);
 
@@ -555,6 +578,151 @@ public:
     }
 };
 
+// define a background class to handle shading of pace zones
+// draws pace zone bands IF zones are defined and the option
+// to draw bonds has been selected
+class PaceHistBackground: public QwtPlotItem
+{
+private:
+    PowerHist *parent;
+
+public:
+    PaceHistBackground(PowerHist *_parent)
+    {
+        setZ(0.0);
+        parent = _parent;
+    }
+
+    virtual int rtti() const
+    {
+        return QwtPlotItem::Rtti_PlotUserItem;
+    }
+
+    virtual void draw(QPainter *painter,
+                      const QwtScaleMap &xMap, const QwtScaleMap &,
+                      const QRectF &rect) const
+    {
+	RideItem *rideItem = parent->rideItem;
+
+    // only for running and swimming activities
+	if (! rideItem || ! (rideItem->isRun || rideItem->isSwim))
+	    return;
+
+	const PaceZones *zones = parent->context->athlete->paceZones(rideItem->isSwim);
+	int zone_range = zones ? zones->whichRange(rideItem->dateTime.date()) : -1;
+
+    // unit conversion factor for imperial units
+    const double speed_factor  = (parent->context->athlete->useMetricUnits ? 1.0 : 0.62137119);
+
+	if (parent->shadePaceZones() && (zone_range >= 0)) {
+	    QList <double> zone_lows = zones->getZoneLows(zone_range);
+	    int num_zones = zone_lows.size();
+	    if (num_zones > 0) {
+		for (int z = 0; z < num_zones; z ++) {
+                    QRectF r = rect;
+
+		    QColor shading_color =
+			paceZoneColor(z, num_zones);
+		    shading_color.setHsv(
+					 shading_color.hue(),
+					 shading_color.saturation() / 4,
+					 shading_color.value()
+					 );
+		    r.setLeft(xMap.transform(zone_lows[z]*speed_factor));
+		    if (z + 1 < num_zones)
+			r.setRight(xMap.transform(zone_lows[z + 1]*speed_factor));
+		    if (r.right() >= r.left())
+			painter->fillRect(r, shading_color);
+		}
+	    }
+	}
+    }
+};
+
+// Zone labels are drawn if pace zone bands are enabled, automatically
+// at the center of the plot
+class PaceHistZoneLabel: public QwtPlotItem
+{
+private:
+    PowerHist *parent;
+    int zone_number;
+    double watts;
+    QwtText text;
+
+public:
+    PaceHistZoneLabel(PowerHist *_parent, int _zone_number)
+    {
+	parent = _parent;
+	zone_number = _zone_number;
+
+	RideItem *rideItem = parent->rideItem;
+
+    // only for running and swimming activities
+	if (! rideItem || ! (rideItem->isRun || rideItem->isSwim))
+	    return;
+
+	const PaceZones *zones = parent->context->athlete->paceZones(rideItem->isSwim);
+	int zone_range = zones ? zones->whichRange(rideItem->dateTime.date()) : -1;
+
+    // unit conversion factor for imperial units
+    const double speed_factor  = (parent->context->athlete->useMetricUnits ? 1.0 : 0.62137119);
+
+	setZ(1.0 + zone_number / 100.0);
+
+	// create new zone labels if we're shading
+	if (parent->shadePaceZones() && (zone_range >= 0)) {
+	    QList <double> zone_lows = zones->getZoneLows(zone_range);
+	    QList <QString> zone_names = zones->getZoneNames(zone_range);
+	    int num_zones = zone_lows.size();
+	    assert(zone_names.size() == num_zones);
+	    if (zone_number < num_zones) {
+                double min = parent->minX;
+                if (zone_lows[zone_number]*speed_factor > min)
+                    min = zone_lows[zone_number]*speed_factor;
+
+                watts =
+		    (
+		     (zone_number + 1 < num_zones) ?
+                     0.5 * (min + zone_lows[zone_number + 1]*speed_factor) :
+		     (
+		      (zone_number > 0) ?
+		      (1.5 * zone_lows[zone_number]*speed_factor - 0.5 * zone_lows[zone_number - 1]*speed_factor) :
+		      2.0 * zone_lows[zone_number]*speed_factor
+		      )
+		     );
+
+		text = QwtText(zone_names[zone_number]);
+		text.setFont(QFont("Helvetica",24, QFont::Bold));
+		QColor text_color = hrZoneColor(zone_number, num_zones);
+		text_color.setAlpha(64);
+		text.setColor(text_color);
+	    }
+	}
+
+    }
+
+    virtual int rtti() const
+    {
+        return QwtPlotItem::Rtti_PlotUserItem;
+    }
+
+    void draw(QPainter *painter,
+	      const QwtScaleMap &xMap, const QwtScaleMap &,
+              const QRectF &rect) const
+    {
+	if (parent->shadePaceZones()) {
+            int x = xMap.transform(watts);
+	    int y = (rect.bottom() + rect.top()) / 2;
+
+	    // the following code based on source for QwtPlotMarker::draw()
+            QRect tr(QPoint(0, 0), text.textSize(painter->font()).toSize());
+	    tr.moveCenter(QPoint(y, -x));
+	    painter->rotate(90);             // rotate text to avoid overlap: this needs to be fixed
+	    text.draw(painter, tr);
+	}
+    }
+};
+
 class HTimeScaleDraw: public QwtScaleDraw
 {
 
