@@ -124,7 +124,7 @@ RideCache::configChanged(qint32 what)
 
             foreach (FieldDefinition field, context->athlete->rideMetadata()->getFields()) 
                 if (field.diary == true) 
-                    calendarText += QString("%1\n").arg(item->metadata_.value(field.name, ""));
+                    calendarText += field.calendarText(item->metadata_.value(field.name, ""));
 
             item->metadata_.insert("Calendar Text", calendarText);
         }
@@ -164,6 +164,8 @@ RideCache::itemChanged()
 void
 RideCache::addRide(QString name, bool dosignal, bool useTempActivities)
 {
+    RideItem *prior = context->ride;
+
     // ignore malformed names
     QDateTime dt;
     if (!RideFile::parseRideFileName(name, &dt)) return;
@@ -201,6 +203,11 @@ RideCache::addRide(QString name, bool dosignal, bool useTempActivities)
 
     if (dosignal) context->notifyRideAdded(last); // here so emitted BEFORE rideSelected is emitted!
 
+    // free up memory from last one, which is no biggie when importing
+    // a single ride, but means we don't exhaust memory when we import
+    // hundreds/thousands of rides in a batch import.
+    if (prior) prior->close();
+
     // notify everyone to select it
     context->ride = last;
     context->notifyRideSelected(last);
@@ -421,7 +428,10 @@ RideCache::getAggregate(QString name, Specification spec, bool useMetricUnits, b
 {
     // get the metric details, so we can convert etc
     const RideMetric *metric = RideMetricFactory::instance().rideMetric(name);
-    if (!metric) return QString("%1 unknown").arg(name);
+    if (!metric) {
+        qDebug()<<"unknown metric:"<<name;
+        return QString("%1 unknown").arg(name);
+    }
 
     // what we will return
     double rvalue = 0;
