@@ -616,10 +616,11 @@ RideFile *RideFileFactory::openRideFile(Context *context, QFile &file,
 
         // Construct the summary text used on the calendar
         QString calendarText;
-        foreach (FieldDefinition field, context->athlete->rideMetadata()->getFields()) {
-            if (field.diary == true && result->getTag(field.name, "") != "") {
-                calendarText += QString("%1\n")
-                        .arg(result->getTag(field.name, ""));
+        if (context) { // will be null in standalone open
+            foreach (FieldDefinition field, context->athlete->rideMetadata()->getFields()) {
+                if (field.diary == true && result->getTag(field.name, "") != "") {
+                    calendarText += field.calendarText(result->getTag(field.name, ""));
+                }
             }
         }
         result->setTag("Calendar Text", calendarText);
@@ -628,7 +629,7 @@ RideFile *RideFileFactory::openRideFile(Context *context, QFile &file,
         result->setTag("Filename", QFileInfo(file.fileName()).fileName());
         result->setTag("Device", result->deviceType());
         result->setTag("File Format", result->fileFormat());
-        result->setTag("Athlete", context->athlete->cyclist);
+        if (context) result->setTag("Athlete", context->athlete->cyclist);
         result->setTag("Year", result->startTime().toString("yyyy"));
         result->setTag("Month", result->startTime().toString("MMMM"));
         result->setTag("Weekday", result->startTime().toString("ddd"));
@@ -654,10 +655,8 @@ RideFile *RideFileFactory::openRideFile(Context *context, QFile &file,
             i->stop -= timeOffset;
         }
 
-        DataProcessorFactory::instance().autoProcess(result);
-
         // calculate derived data series -- after data fixers applied above
-        result->recalculateDerivedSeries();
+        if (context) result->recalculateDerivedSeries();
 
         // what data is present - after processor in case 'derived' or adjusted
         QString flags;
@@ -1752,7 +1751,7 @@ RideFile::recalculateDerivedSeries(bool force)
 
     // wheelsize - use meta, then config then drop to 2100
     double wheelsize = getTag(tr("Wheelsize"), "0.0").toDouble();
-    if (wheelsize == 0) wheelsize = appsettings->value(this, GC_WHEELSIZE, 2100).toInt();
+    if (wheelsize == 0) wheelsize = appsettings->cvalue(context->athlete->cyclist, GC_WHEELSIZE, 2100).toInt();
     wheelsize /= 1000.00f; // need it in meters
 
     // last point looked at
@@ -2570,3 +2569,83 @@ RideFileInterval::isBest() const
     return QRegExp(best).exactMatch(name); 
 }
 
+
+// ride data is referenced with symbols in upper case to make 
+// it clear that this is raw data
+static struct {
+    QString symbol;
+    RideFile::SeriesType series;
+} seriesSymbolTable[] = {
+
+	{ "SECS", RideFile::secs },
+	{ "CADENCE", RideFile::cad },
+	{ "CADENCED", RideFile::cadd },
+	{ "HEARTRATE", RideFile::hr },
+	{ "HEARTRATED", RideFile::hrd },
+	{ "DISTANCE", RideFile::km },
+	{ "SPEED", RideFile::kph },
+	{ "SPEEDD", RideFile::kphd },
+	{ "TORQUE", RideFile::nm },
+	{ "TORQUED", RideFile::nmd },
+	{ "POWER", RideFile::watts },
+	{ "POWERD", RideFile::wattsd },
+	{ "ALTITUDE", RideFile::alt },
+	{ "LON", RideFile::lon },
+	{ "LAT", RideFile::lat },
+	{ "HEADWIND", RideFile::headwind },
+	{ "SLOPE", RideFile::slope },
+	{ "TEMPERATURE", RideFile::temp },
+	{ "BALANCE", RideFile::lrbalance },
+	{ "LEFTEFFECTIVENESS", RideFile::lte },
+	{ "RIGHTEFFECTIVENESS", RideFile::rte },
+	{ "LEFTSMOOTHNESS", RideFile::lps },
+	{ "RIGHTSMOOTHNESS", RideFile::rps },
+	{ "SMO2", RideFile::smo2 },
+	{ "THB", RideFile::thb },
+	{ "RUNVERT", RideFile::rvert },
+	{ "RUNCADENCE", RideFile::rcad },
+	{ "RUNCONTACT", RideFile::rcontact },
+	{ "LEFTPCO", RideFile::lpco },
+	{ "RIGHTPCO", RideFile::rpco },
+	{ "LEFTPPB", RideFile::lppb },
+	{ "RIGHTPPB", RideFile::rppb },
+	{ "LEFTPPE", RideFile::lppe },
+	{ "RIGHTPPE", RideFile::rppe },
+	{ "LEFTPPPB", RideFile::lpppb },
+	{ "RIGHTPPPB", RideFile::rpppb },
+	{ "LEFTPPPE", RideFile::lpppe },
+	{ "RIGHTPPPE", RideFile::rpppe },
+	{ "", RideFile::none  },
+};
+
+QStringList 
+RideFile::symbols()
+{
+    // list of valid symbols
+    QStringList returning;
+    for(int i=0; seriesSymbolTable[i].series != none; i++)
+        returning << seriesSymbolTable[i].symbol;
+
+    return returning;
+}
+
+RideFile::SeriesType RideFile::seriesForSymbol(QString symbol)
+{
+    // get type for symbol
+    for(int i=0; seriesSymbolTable[i].series != none; i++)
+        if (seriesSymbolTable[i].symbol == symbol)
+            return seriesSymbolTable[i].series;
+
+    return none;
+}
+
+QString 
+RideFile::symbolForSeries(SeriesType series)
+{
+    // get type for symbol
+    for(int i=0; seriesSymbolTable[i].series != none; i++)
+        if (seriesSymbolTable[i].series == series)
+            return seriesSymbolTable[i].symbol;
+
+    return "";
+}
