@@ -34,6 +34,8 @@ class RideCache;
 class RideCacheModel;
 class IntervalItem;
 class Context;
+class UserData;
+class ComparePane;
 
 Q_DECLARE_METATYPE(RideItem*)
 
@@ -49,12 +51,14 @@ class RideItem : public QObject
         friend class ::RideCache;
         friend class ::RideCacheModel;
         friend class ::IntervalItem;
+        friend class ::UserData;
+        friend class ::ComparePane;
 
         // ridefile
         RideFile *ride_;
         RideFileCache *fileCache_;
 
-        // precomputed metrics
+        // precomputed metrics & user overrides
         QVector<double> metrics_;
 
         // metadata (used by navigator)
@@ -64,6 +68,9 @@ class RideItem : public QObject
         QList<IntervalItem*> intervals_;
         QStringList errors_;
 
+        // userdata cache
+        QMap<QString, QVector<double> > userCache;
+
         unsigned long metaCRC();
 
     public slots:
@@ -88,6 +95,9 @@ class RideItem : public QObject
         // set from another, e.g. during load of rideDB.json
         void setFrom(RideItem&);
 
+        // record of any overrides, used by formula "isset" function
+        QStringList overrides_;
+
         // set metric values e.g. when working with intervals
         void setFrom(QHash<QString, RideMetricPtr>);
 
@@ -106,6 +116,7 @@ class RideItem : public QObject
 
         // access the metadata
         QString getText(QString name, QString fallback) { return metadata_.value(name, fallback); }
+        bool hasText(QString name) { return metadata_.contains(name); }
 
         // get at the first class data
         QString path;
