@@ -661,6 +661,9 @@ FormField::metadataFlush()
             }
         }
         break;
+    case FIELD_CHECKBOX :
+        text = ((QCheckBox *)widget)->isChecked() ? "1" : "0";
+        break;
     case FIELD_DATE : text = ((QDateEdit*)widget)->date().toString("dd.MM.yyyy"); break;
     case FIELD_TIME : text = ((QTimeEdit*)widget)->time().toString("hh:mm:ss.zzz"); break;
     }
@@ -725,8 +728,7 @@ FormField::metadataFlush()
     QString calendarText;
     foreach (FieldDefinition field, meta->getFields()) {
         if (field.diary == true) {
-            calendarText += QString("%1\n")
-                    .arg(ourRideItem->ride()->getTag(field.name, ""));
+            calendarText += field.calendarText(ourRideItem->ride()->getTag(field.name, ""));
         }
     }
     ourRideItem->ride()->setTag("Calendar Text", calendarText);
@@ -1152,6 +1154,24 @@ FieldDefinition::getCompleter(QObject *parent)
     return completer;
 }
 
+QString
+FieldDefinition::calendarText(QString value)
+{
+    switch (type) {
+    case FIELD_INTEGER:
+    case FIELD_DOUBLE:
+    case FIELD_DATE:
+    case FIELD_TIME:
+    case FIELD_CHECKBOX:
+        return QString("%1: %2\n").arg(name).arg(value);
+    case FIELD_TEXT:
+    case FIELD_TEXTBOX:
+    case FIELD_SHORTTEXT:
+    default:
+        return QString("%1\n").arg(value);
+    }
+}
+
 unsigned long
 KeywordDefinition::fingerprint(QList<KeywordDefinition> list)
 {
