@@ -19,11 +19,14 @@
 #include "Athlete.h"
 #include "Context.h"
 #include "Colors.h"
+#include "RideCache.h"
+#include "RideCacheModel.h"
 #include "RideItem.h"
 #include "RideNavigator.h"
 #include "RideNavigatorProxy.h"
 #include "SearchFilterBox.h"
 #include "TabView.h"
+#include "HelpWhatsThis.h"
 
 #include <QtGui>
 #include <QString>
@@ -46,6 +49,7 @@ RideNavigator::RideNavigator(Context *context, bool mainwindow) : context(contex
     fontHeight = QFontMetrics(QFont()).height();
     ColorEngine ce(context);
     reverseColor = ce.reverseColor;
+    currentItem = NULL;
 
     init = false;
 
@@ -54,11 +58,8 @@ RideNavigator::RideNavigator(Context *context, bool mainwindow) : context(contex
     if (mainwindow) mainLayout->setContentsMargins(0,0,0,0);
     else mainLayout->setContentsMargins(2,2,2,2); // so we can resize!
 
-    context->athlete->sqlModel->select();
-    while (context->athlete->sqlModel->canFetchMore(QModelIndex())) context->athlete->sqlModel->fetchMore(QModelIndex());
-
     searchFilter = new SearchFilter(this);
-    searchFilter->setSourceModel(context->athlete->sqlModel); // filter out/in search results
+    searchFilter->setSourceModel(context->athlete->rideCache->model()); // filter out/in search results
 
     groupByModel = new GroupByModel(this);
     groupByModel->setSourceModel(searchFilter);
@@ -67,12 +68,12 @@ RideNavigator::RideNavigator(Context *context, bool mainwindow) : context(contex
     sortModel->setSourceModel(groupByModel);
     sortModel->setDynamicSortFilter(true);
 
-#ifdef GC_HAVE_LUCENE
     if (!mainwindow) {
         searchFilterBox = new SearchFilterBox(this, context, false);
         mainLayout->addWidget(searchFilterBox);
+        HelpWhatsThis *searchHelp = new HelpWhatsThis(searchFilterBox);
+        searchFilterBox->setWhatsThis(searchHelp->getWhatsThisText(HelpWhatsThis::SearchFilterBox));
     }
-#endif
 
     // get setup
     tableView = new RideTreeView;
@@ -105,43 +106,53 @@ RideNavigator::RideNavigator(Context *context, bool mainwindow) : context(contex
     tableView->setAcceptDrops(true);
     tableView->setColumnWidth(1, 100);
 
+    HelpWhatsThis *helpTableView = new HelpWhatsThis(tableView);
+    if (mainwindow)
+        tableView->setWhatsThis(helpTableView->getWhatsThisText(HelpWhatsThis::SideBarRidesView_Rides));
+    else
+        tableView->setWhatsThis(helpTableView->getWhatsThisText(HelpWhatsThis::ChartDiary_Navigator));
+
     // good to go
     tableView->show();
     resetView();
 
-    // refresh when database is updated
-    connect(context->athlete->metricDB, SIGNAL(dataChanged()), this, SLOT(refresh()));
-
     // refresh when config changes (metric/imperial?)
-    connect(context, SIGNAL(configChanged()), this, SLOT(configChanged()));
+    connect(context, SIGNAL(configChanged(qint32)), this, SLOT(configChanged(qint32)));
+
     // refresh when rides added/removed
     connect(context, SIGNAL(rideAdded(RideItem*)), this, SLOT(refresh()));
-    connect(context, SIGNAL(rideDeleted(RideItem*)), this, SLOT(refresh()));
-    connect(context->athlete->rideTreeWidget(), SIGNAL(itemSelectionChanged()), this, SLOT(rideTreeSelectionChanged()));
-    // selection of a ride by double clicking it, we need to update the ride list
-    connect(tableView, SIGNAL(doubleClicked(QModelIndex)), this, SLOT(selectRide(QModelIndex)));
-    connect(tableView->selectionModel(), SIGNAL(selectionChanged(QItemSelection, QItemSelection)), this, SLOT(cursorRide()));
+    connect(context, SIGNAL(rideDeleted(RideItem*)), this, SLOT(rideDeleted(RideItem*)));
+
     // user selected a ride on the ride list, we should reflect that too..
+    connect(tableView, SIGNAL(rowSelected(QItemSelection)), this, SLOT(selectionChanged(QItemSelection)));
+
+    // user hit return or double clicked a ride !
+    connect(tableView, SIGNAL(doubleClicked(QModelIndex)), this, SLOT(selectRide(QModelIndex)));
+
     // user moved columns
     connect(tableView->header(), SIGNAL(sectionMoved(int,int,int)), this, SLOT(columnsChanged()));
-    connect(tableView->header(), SIGNAL(sectionResized(int,int,int)), this, SLOT(columnsChanged()));
+    connect(tableView->header(), SIGNAL(sectionResized(int,int,int)), this, SLOT(columnsResized(int, int, int)));
+
     // user sorted by column
-    connect(tableView->header(), SIGNAL(sortIndicatorChanged(int, Qt::SortOrder)), this, SLOT(selectRow()));
+    connect(tableView->header(), SIGNAL(sortIndicatorChanged(int, Qt::SortOrder)), this, SLOT(cursorRide()));
     connect(tableView,SIGNAL(customContextMenuRequested(const QPoint &)), this, SLOT(showTreeContextMenuPopup(const QPoint &)));
     connect(tableView->header(), SIGNAL(sortIndicatorChanged(int,Qt::SortOrder)), this, SLOT(setSortBy(int,Qt::SortOrder)));
 
-#ifdef GC_HAVE_LUCENE
+    // repaint etc when background refresh is working
+    connect(context, SIGNAL(refreshStart()), this, SLOT(backgroundRefresh()));
+    connect(context, SIGNAL(refreshEnd()), this, SLOT(backgroundRefresh()));
+    connect(context, SIGNAL(refreshUpdate(QDate)), this, SLOT(backgroundRefresh())); // we might miss 1st one
+
     if (!mainwindow) {
         connect(searchFilterBox, SIGNAL(searchResults(QStringList)), this, SLOT(searchStrings(QStringList)));
         connect(searchFilterBox, SIGNAL(searchClear()), this, SLOT(clearSearch()));
     }
-#endif
 
     // we accept drag and drop operations
     setAcceptDrops(true);
 
     // lets go
-    configChanged();
+    configChanged(CONFIG_APPEARANCE | CONFIG_NOTECOLOR | CONFIG_FIELDS);
 }
 
 RideNavigator::~RideNavigator()
@@ -151,7 +162,7 @@ RideNavigator::~RideNavigator()
 }
 
 void
-RideNavigator::configChanged()
+RideNavigator::configChanged(qint32 state)
 {
     ColorEngine ce(context);
     fontHeight = QFontMetrics(QFont()).height();
@@ -165,9 +176,9 @@ RideNavigator::configChanged()
             tableView->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
         else 
             tableView->setVerticalScrollBarPolicy(Qt::ScrollBarAsNeeded);
-        if (appsettings->value(this, GC_RIDEHEAD, true).toBool() == false)
-            tableView->header()->hide();
-        else 
+        //if (appsettings->value(this, GC_RIDEHEAD, true).toBool() == false)
+            //tableView->header()->hide();
+        //else 
             tableView->header()->show();
 
         tableView->header()->setStyleSheet(
@@ -180,20 +191,32 @@ RideNavigator::configChanged()
 
 #endif
 
+    // if the fields changed we need to reset indexes etc
+    if (state & CONFIG_FIELDS) resetView();
+
+    refresh();
+}
+
+void
+RideNavigator::rideDeleted(RideItem*item)
+{
+    if (currentItem == item) currentItem = NULL;
     refresh();
 }
 
 void
 RideNavigator::refresh()
 {
-    context->athlete->sqlModel->select();
-    while (context->athlete->sqlModel->canFetchMore(QModelIndex()))
-        context->athlete->sqlModel->fetchMore(QModelIndex());
-
     active=false;
 
     setWidth(geometry().width());
-    rideTreeSelectionChanged();
+    cursorRide();
+}
+
+void
+RideNavigator::backgroundRefresh()
+{
+    tableView->doItemsLayout();
 }
 
 void
@@ -245,7 +268,7 @@ RideNavigator::resetView()
         QString converted = QTextEdit(factory.rideMetric(factory.metricName(i))->name()).toPlainText();
 
         // from sql column name to friendly metric name
-        nameMap.insert(QString("X%1").arg(factory.metricName(i)), converted);
+        nameMap.insert(QString("%1").arg(factory.metricName(i)), converted);
 
         // from (english) internalName to (translated) Name
         internalNameMap.insert(factory.rideMetric(factory.metricName(i))->internalName(), converted);
@@ -258,7 +281,7 @@ RideNavigator::resetView()
     SpecialFields sp; // all the special fields are in here...
     foreach(FieldDefinition field, context->athlete->rideMetadata()->getFields()) {
         if (!sp.isMetric(field.name) && (field.type < 5 || field.type == 7)) {
-            nameMap.insert(QString("Z%1").arg(sp.makeTechName(field.name)), sp.displayName(field.name));
+            nameMap.insert(QString("%1").arg(sp.makeTechName(field.name)), sp.displayName(field.name));
             internalNameMap.insert(field.name, sp.displayName(field.name));
         }
     }
@@ -274,6 +297,7 @@ RideNavigator::resetView()
     // setup the logical heading list
     for (int i=0; i<tableView->header()->count(); i++) {
         QString friendly, techname = sortModel->headerData(i, Qt::Horizontal).toString();
+
         if ((friendly = nameMap.value(techname, "unknown")) != "unknown") {
             sortModel->setHeaderData(i, Qt::Horizontal, friendly);
             logicalHeadings << friendly;
@@ -321,7 +345,10 @@ RideNavigator::resetView()
     resizeEvent(NULL);
 
     // Select the current ride
-    rideTreeSelectionChanged();
+    cursorRide();
+
+    // get height
+    tableView->doItemsLayout();
 
     columnsChanged();
 }
@@ -341,92 +368,9 @@ void RideNavigator::clearSearch()
 
 void RideNavigator::setWidth(int x)
 {
-    if (init == false) return;
+    // use helper function
+    setColumnWidth(x, false);
 
-    active = true;
-
-#if !defined (Q_OS_MAC) || (defined (Q_OS_MAC) && (QT_VERSION < 0x050000)) // on QT5 the scrollbars have no width
-    if (tableView->verticalScrollBar()->isVisible())
-        x -= tableView->verticalScrollBar()->width()
-             + 0 ; // !! no longer account for content margins of 3,3,3,3 was + 6
-#else // we're on a mac with QT5 .. so dodgy way of spotting preferences for scrollbars...
-      // this is a nasty hack, to see if the 'always on' preference for scrollbars is set we
-      // look at the scrollbar width which is 15 in this case (it is 16 when they 'appear' when
-      // needed. No doubt this will change over time and need to be fixed by referencing the
-      // Mac system preferences via an NSScroller - but that will be a massive hassle.
-      if (tableView->verticalScrollBar()->isVisible() && tableView->verticalScrollBar()->width() == 15)
-          x -= tableView->verticalScrollBar()->width() + 0 ; 
-#endif
-
-    // take the margins into accopunt top
-    x -= mainLayout->contentsMargins().left() + mainLayout->contentsMargins().right();
-
-    // ** NOTE **
-    // When iterating over the section headings we
-    // always use the tableview not the sortmodel
-    // so we can skip over the virtual column 0
-    // which is used to group by, is visible but
-    // must have a width of 0. This is why all
-    // the for loops start with i=1
-    tableView->setColumnWidth(0,0); // in case use grabbed it
-
-    // is it narrower than the headings?
-    int headwidth=0;
-    int n=0;
-    for (int i=1; i<tableView->header()->count(); i++)
-        if (tableView->header()->isSectionHidden(i) == false) {
-            headwidth += tableView->columnWidth(i);
-            n++;
-        }
-
-    // headwidth is no, x is to-be width
-    // we need to 'stretch' the sections 
-    // proportionally to fit into new
-    // layout
-    int setwidth=0;
-    int newwidth=0;
-    for (int i=1; i<tableView->header()->count(); i++) {
-        if (tableView->header()->isSectionHidden(i) == false) {
-            newwidth = (double)((((double)tableView->columnWidth(i)/(double)headwidth)) * (double)x); 
-            if (newwidth < 20) newwidth = 20;
-            QString columnName = tableView->model()->headerData(i, Qt::Horizontal).toString();
-            if (columnName == "*") newwidth = 0;
-            tableView->setColumnWidth(i, newwidth);
-            setwidth += newwidth;
-        }
-    }
-
-    // UGH. Now account for the fact that the smaller columns
-    //      didn't take their fair share of a negative resize
-    //      so we need to snip off from the larger columns.
-    if (setwidth != x) {
-        // how many columns we got to snip from?
-        int colsleft = 0;
-        for (int i=1; i<tableView->header()->count(); i++)
-            if (tableView->header()->isSectionHidden(i) == false && tableView->columnWidth(i)>20)
-                colsleft++;
-
-        // run through ... again.. snipping off some pixels
-        if (colsleft) {
-            int snip = (setwidth-x) / colsleft; //could be negative too
-            for (int i=1; i<tableView->header()->count(); i++) {
-                if (tableView->header()->isSectionHidden(i) == false && tableView->columnWidth(i)>20) {
-                    tableView->setColumnWidth(i, tableView->columnWidth(i)-snip);
-                    setwidth -= snip;
-                }
-            }
-        }
-    }
-
-    if (setwidth < x)
-        delegate->setWidth(pwidth=setwidth);
-    else
-        delegate->setWidth(pwidth=x);
-
-    // make the scrollbars go away
-    tableView->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
-
-    active = false;
 }
 
 // make sure the columns are all neat and tidy when the ride navigator is shown
@@ -466,38 +410,16 @@ RideNavigator::setGroupByColumnName(QString name)
 void
 RideNavigator::columnsChanged()
 {
-    if (active == true) return;
-    active = true;
+    // do the work - (column changed, but no "inWidget" column resize)
+    calcColumnsChanged(false);
+}
 
-    visualHeadings.clear(); // they have moved
+void
+RideNavigator::columnsResized(int logicalIndex, int oldSize, int newSize)
+{
 
-    // get the names used
-    for (int i=0; i<tableView->header()->count(); i++) {
-        if (tableView->header()->isSectionHidden(tableView->header()->logicalIndex(i)) != true) {
-            int index = tableView->header()->logicalIndex(i);
-            visualHeadings << logicalHeadings[index];
-        }
-    }
-    // write to config
-    QString headings;
-    foreach(QString name, visualHeadings)
-        headings += name + "|";
-
-    _columns = headings;
-
-    // get column widths
-    QString widths;
-    for (int i=0; i<tableView->header()->count(); i++) {
-        int index = tableView->header()->logicalIndex(i);
-        if (tableView->header()->isSectionHidden(index) != true) {
-           widths += QString("%1|").arg(tableView->columnWidth(index));
-        }
-    }
-
-    // clean up
-    active = false;
-    setWidth(geometry().width()); // calculate width...
-    _widths = widths;
+    // do the work - resize only
+    calcColumnsChanged(true, logicalIndex, oldSize, newSize);
 }
 
 bool
@@ -506,7 +428,7 @@ RideNavigator::eventFilter(QObject *object, QEvent *e)
     // not for the table?
     if (object != (QObject *)tableView) return false;
 
-    // what happenned?
+    // what happnned?
     switch(e->type())
     {
         case QEvent::ContextMenu:
@@ -641,7 +563,7 @@ RideNavigator::setGroupByColumn()
 
     // reselect current ride - since selectionmodel
     // is changed by setGroupBy()
-    rideTreeSelectionChanged();
+    cursorRide();
 }
 
 void
@@ -651,8 +573,196 @@ RideNavigator::setSortBy(int index, Qt::SortOrder order)
     _sortByOrder = static_cast<int>(order);
 }
 
+
+void
+RideNavigator::calcColumnsChanged(bool resized, int logicalIndex, int oldSize, int newSize ) {
+
+    // double use - for "changing" and "only resizing" of the columns
+
+    if (active == true) return;
+    active = true;
+
+    visualHeadings.clear(); // they have moved
+
+    // get the names used
+    for (int i=0; i<tableView->header()->count(); i++) {
+        if (tableView->header()->isSectionHidden(tableView->header()->logicalIndex(i)) != true) {
+            int index = tableView->header()->logicalIndex(i);
+            visualHeadings << logicalHeadings[index];
+        }
+    }
+    // write to config
+    QString headings;
+    foreach(QString name, visualHeadings)
+        headings += name + "|";
+
+    _columns = headings;
+
+    // correct width and store result
+    setColumnWidth(geometry().width(), resized, logicalIndex, oldSize, newSize); // calculate width...
+
+    // get column widths
+    QString widths;
+    for (int i=0; i<tableView->header()->count(); i++) {
+        int index = tableView->header()->logicalIndex(i);
+        if (tableView->header()->isSectionHidden(index) != true) {
+           widths += QString("%1|").arg(tableView->columnWidth(index));
+        }
+    }
+    _widths = widths;
+
+    active = false;
+}
+
+void
+RideNavigator::setColumnWidth(int x, bool resized, int logicalIndex, int oldWidth, int newWidth) {
+
+    // double use - for use after any change (e.g. widget size,..) "changing" and "only resizing" of the columns
+
+    if (init == false) return;
+
+    active = true;
+
+#if !defined (Q_OS_MAC) || (defined (Q_OS_MAC) && (QT_VERSION < 0x050000)) // on QT5 the scrollbars have no width
+    if (tableView->verticalScrollBar()->isVisible())
+        x -= tableView->verticalScrollBar()->width()
+                + 0 ; // !! no longer account for content margins of 3,3,3,3 was + 6
+#else // we're on a mac with QT5 .. so dodgy way of spotting preferences for scrollbars...
+    // this is a nasty hack, to see if the 'always on' preference for scrollbars is set we
+    // look at the scrollbar width which is 15 in this case (it is 16 when they 'appear' when
+    // needed. No doubt this will change over time and need to be fixed by referencing the
+    // Mac system preferences via an NSScroller - but that will be a massive hassle.
+    if (tableView->verticalScrollBar()->isVisible() && tableView->verticalScrollBar()->width() == 15)
+        x -= tableView->verticalScrollBar()->width() + 0 ;
+#endif
+
+    // take the margins into account top
+    x -= mainLayout->contentsMargins().left() + mainLayout->contentsMargins().right();
+
+    // ** NOTE **
+    // When iterating over the section headings we
+    // always use the tableview not the sortmodel
+    // so we can skip over the virtual column 0
+    // which is used to group by, is visible but
+    // must have a width of 0. This is why all
+    // the for loops start with i=1
+    tableView->setColumnWidth(0,0); // in case use grabbed it
+
+    // is it narrower than the headings?
+    int headwidth=0;
+    int n=0;
+    for (int i=1; i<tableView->header()->count(); i++)
+        if (tableView->header()->isSectionHidden(i) == false) {
+            headwidth += tableView->columnWidth(i);
+            n++;
+        }
+
+    if (!resized) {
+
+        // headwidth is no, x is to-be width
+        // we need to 'stretch' the sections
+        // proportionally to fit into new
+        // layout
+        int setwidth=0;
+        int newwidth=0;
+        for (int i=1; i<tableView->header()->count(); i++) {
+            if (tableView->header()->isSectionHidden(i) == false) {
+                newwidth = (double)((((double)tableView->columnWidth(i)/(double)headwidth)) * (double)x);
+                if (newwidth < 20) newwidth = 20;
+                QString columnName = tableView->model()->headerData(i, Qt::Horizontal).toString();
+                if (columnName == "*") newwidth = 0;
+                tableView->setColumnWidth(i, newwidth);
+                setwidth += newwidth;
+            }
+        }
+
+        // UGH. Now account for the fact that the smaller columns
+        //      didn't take their fair share of a negative resize
+        //      so we need to snip off from the larger columns.
+        if (setwidth != x) {
+            // how many columns we got to snip from?
+            int colsleft = 0;
+            for (int i=1; i<tableView->header()->count(); i++)
+                if (tableView->header()->isSectionHidden(i) == false && tableView->columnWidth(i)>20)
+                    colsleft++;
+
+            // run through ... again.. snipping off some pixels
+            if (colsleft) {
+                int snip = (setwidth-x) / colsleft; //could be negative too
+                for (int i=1; i<tableView->header()->count(); i++) {
+                    if (tableView->header()->isSectionHidden(i) == false && tableView->columnWidth(i)>20) {
+                        tableView->setColumnWidth(i, tableView->columnWidth(i)-snip);
+                        setwidth -= snip;
+                    }
+                }
+            }
+        }
+
+        if (setwidth < x)
+            delegate->setWidth(pwidth=setwidth);
+        else
+            delegate->setWidth(pwidth=x);
+
+    } else {
+
+        // columns are resized - for each affected column this function is called
+        // and makes sure that
+        // a) nothing gets smaller than 20 and
+        // b) last section is not moved over the right border / does not fill the widget to the right border
+
+        // first step: make sure that the current column got smaller than 20 by resizing
+        if (newWidth < 20) {
+            tableView->setColumnWidth(logicalIndex, oldWidth);
+            // correct the headwidth by the added space
+            headwidth += (oldWidth - newWidth);
+        }
+
+        // get the index of the most right column (since here all further resizing will start)
+        int visIndex = 0;
+        // find the most right visible column
+        for (int i=1; i<tableView->header()->count(); i++) {
+            if (tableView->header()->isSectionHidden(i) == false &&
+                    tableView->header()->visualIndex(i) > visIndex)
+                visIndex = tableView->header()->visualIndex(i);
+        }
+
+        if (headwidth > x) {
+            // now make sure that no column disappears right border of the table view
+            // by taking the overlapping part "cut" from last column(s)
+            int cut = headwidth - x;
+             // now resize, but not smaller than 20 (from right to left of Visible Columns)
+            while (cut >0 && visIndex > 0) {
+                int logIndex = tableView->header()->logicalIndex(visIndex);
+                int k = tableView->columnWidth(logIndex);
+                if (k - cut >= 20) {
+                    tableView->setColumnWidth(logIndex, k-cut);
+                    cut = 0;
+                } else {
+                    tableView->setColumnWidth(logIndex, 20);
+                    cut -= (k-20);
+                }
+                visIndex--;
+            }
+        } else {
+            // since QT on fast mouse moves resizes more columns then expected
+            // give all available space to the last visible column
+            int logIndex = tableView->header()->logicalIndex(visIndex);
+            int k = tableView->columnWidth(logIndex);
+            tableView->setColumnWidth(logIndex, (k+x-headwidth));
+        }
+    }
+
+    // make the scrollbars go away
+    tableView->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
+    active = false;
+
+}
+
+
+
+
 //
-// This function is called for every row in the metricDB
+// This function is called for every row in the ridecache
 // and wants to know what group string or 'name' you want
 // to put this row into. It is passed the heading value
 // as a string, and the row value for this column.
@@ -833,74 +943,86 @@ RideNavigator::showColumnChooser()
     selector->show();
 }
 
-
-// user double clicked a ride, we need to update the main window ride list
+// user selected a different ride somewhere else, we need to align with that
 void
-RideNavigator::selectRide(const QModelIndex &index)
+RideNavigator::setRide(RideItem*rideItem)
 {
-    QModelIndex fileIndex = tableView->model()->index(index.row(), 2, index.parent()); // column 2 for filename ?
+    if (currentItem == rideItem) return;
 
-    QString filename = tableView->model()->data(fileIndex, Qt::DisplayRole).toString();
-    context->athlete->selectRideFile(filename);
-}
+    for (int i=0; i<tableView->model()->rowCount(); i++) {
 
-// user cursor moved to ride
-void
-RideNavigator::cursorRide()
-{
-    if (active ==  true) return;
-    else active = true;
-    selectRide(tableView->currentIndex());
-    active = false;
-}
+        QModelIndex group = tableView->model()->index(i,0,QModelIndex());
+        for (int j=0; j<tableView->model()->rowCount(group); j++) {
 
-// fixup selection lost when columns sorted etc
-void
-RideNavigator::selectRow()
-{
-    // this is fugly and either a bug in QtreeView sorting
-    // or a bug in our QAbstractProxyModel.
-    rideTreeSelectionChanged(); // reset from ridelist
-}
+            QString fileName = tableView->model()->data(tableView->model()->index(j,3, group), Qt::DisplayRole).toString();
+            if (fileName == rideItem->fileName) {
+                // we set current index to column 2 (date/time) since we can be guaranteed it is always show (all others are removable)
+                QItemSelection row(tableView->model()->index(j,0,group),
+                tableView->model()->index(j,tableView->model()->columnCount()-1, group));
+                tableView->selectionModel()->select(row, QItemSelectionModel::Rows | QItemSelectionModel::ClearAndSelect);
+                tableView->selectionModel()->setCurrentIndex(tableView->model()->index(j,0,group), QItemSelectionModel::NoUpdate);
+                tableView->scrollTo(tableView->model()->index(j,3,group), QAbstractItemView::PositionAtCenter);
 
-// the main window ride list changed, we need to reflect it
+                currentItem = rideItem;
+                repaint();
+                active = false;
+                return;
+            }
+        }
+    }
+}
+
+void
+RideNavigator::selectionChanged(QItemSelection selected)
+{
+    QModelIndex ref = selected.indexes().first();
+    QModelIndex fileIndex = tableView->model()->index(ref.row(), 3, ref.parent());
+    QString filename = tableView->model()->data(fileIndex, Qt::DisplayRole).toString();
+
+    // lets make sure we know what we've selected, so we don't
+    // select it twice
+    foreach(RideItem *item, context->athlete->rideCache->rides()) {
+       if (item->fileName == filename) {
+           currentItem = item;
+           break;
+       }
+    }
+
+    // lets notify others
+    context->athlete->selectRideFile(filename);
+
+}
+
 void
-RideNavigator::rideTreeSelectionChanged()
+RideNavigator::selectRide(const QModelIndex &index)
 {
-    if (active == true) return;
-    else active = true;
-
-    QTreeWidgetItem *which;
-    if (context->athlete->rideTreeWidget()->selectedItems().count())
-        which = context->athlete->rideTreeWidget()->selectedItems().first();
-    else // no rides slected
-        which = NULL;
-
-    if (which && which->type() == RIDE_TYPE) {
-        RideItem *rideItem = static_cast<RideItem *>(which);
-
-        // now find rideItem->fileName in the model
-        // and then select it in the view!
-        for (int i=0; i<tableView->model()->rowCount(); i++) {
-
-            QModelIndex group = tableView->model()->index(i,0,QModelIndex());
-            for (int j=0; j<tableView->model()->rowCount(group); j++) {
-
-                QString fileName = tableView->model()->data(tableView->model()->index(j,2, group), Qt::DisplayRole).toString();
-                if (fileName == rideItem->fileName) {
-                    // we set current index to column 2 (date/time) since we can be guaranteed it is always show (all others are removable)
-                    QItemSelection row(tableView->model()->index(j,0,group),
-                                       tableView->model()->index(j,tableView->model()->columnCount()-1, group));
-                    tableView->selectionModel()->select(row, QItemSelectionModel::Rows | QItemSelectionModel::ClearAndSelect);
-                    tableView->selectionModel()->setCurrentIndex(tableView->model()->index(j,0,group), QItemSelectionModel::NoUpdate);
-                    tableView->scrollTo(tableView->model()->index(j,3,group), QAbstractItemView::PositionAtCenter);
-                    active = false;
-                    return;
-                }
+    // we don't use this at present, but hitting return
+    // or double clicking a ride will cause this to get called....
+    QModelIndex fileIndex = tableView->model()->index(index.row(), 3, index.parent()); // column 2 for filename ?
+    QString filename = tableView->model()->data(fileIndex, Qt::DisplayRole).toString();
+
+    // do nothing .. but maybe later do something ?
+    //context->athlete->selectRideFile(filename);
+}
+
+void
+RideNavigator::cursorRide()
+{
+    if (currentItem == NULL) return;
+
+    // find our ride and scroll to it
+    for (int i=0; i<tableView->model()->rowCount(); i++) {
+
+        QModelIndex group = tableView->model()->index(i,0,QModelIndex());
+        for (int j=0; j<tableView->model()->rowCount(group); j++) {
+            QString fileName = tableView->model()->data(tableView->model()->index(j,2, group), Qt::UserRole+1).toString();
+            if (fileName == currentItem->fileName) {
+                // we set current index to column 2 (date/time) since we can be guaranteed it is always show (all others are removable)
+                tableView->scrollTo(tableView->model()->index(j,3,group));
+                return;
             }
         }
     }
-    active = false;
 }
 
 // Drag and drop columns from the chooser...
@@ -939,6 +1061,7 @@ void NavigatorCellDelegate::commitAndCloseEditor() { }
 void NavigatorCellDelegate::setEditorData(QWidget *, const QModelIndex &) const { }
 void NavigatorCellDelegate::updateEditorGeometry(QWidget *, const QStyleOptionViewItem &, const QModelIndex &) const {}
 void NavigatorCellDelegate::setModelData(QWidget *, QAbstractItemModel *, const QModelIndex &) const { }
+bool NavigatorCellDelegate::helpEvent(QHelpEvent*, QAbstractItemView*, const QStyleOptionViewItem&, const QModelIndex&) { return true; }
 
 QSize NavigatorCellDelegate::sizeHint(const QStyleOptionViewItem & /*option*/, const QModelIndex &index) const 
 {
@@ -962,13 +1085,15 @@ void NavigatorCellDelegate::paint(QPainter *painter, const QStyleOptionViewItem
     // state of item
     bool hover = option.state & QStyle::State_MouseOver;
     bool selected = option.state & QStyle::State_Selected;
+    bool focus = option.state & QStyle::State_HasFocus;
+    //bool isRun = rideNavigator->tableView->model()->data(index, Qt::UserRole+2).toBool();
 
     // format the cell depending upon what it is...
     QString columnName = rideNavigator->tableView->model()->headerData(index.column(), Qt::Horizontal).toString();
     const RideMetric *m;
     QString value;
 
-    // are we a selected cell ? need to paint acordingly
+    // are we a selected cell ? need to paint accordingly
     //bool selected = false;
     //if (rideNavigator->tableView->selectionModel()->selectedIndexes().count()) { // zero if no rides in list
         //if (rideNavigator->tableView->selectionModel()->selectedIndexes().value(0).row() == index.row())
@@ -976,28 +1101,12 @@ void NavigatorCellDelegate::paint(QPainter *painter, const QStyleOptionViewItem
     //}
 
     if ((m=rideNavigator->columnMetrics.value(columnName, NULL)) != NULL) {
-        // format as a metric
 
         // get double from sqlmodel
-        double metricValue = index.model()->data(index, Qt::DisplayRole).toDouble();
+        value = index.model()->data(index, Qt::DisplayRole).toString();
 
-        if (metricValue) {
-            // metric / imperial converstion
-            metricValue *= (rideNavigator->context->athlete->useMetricUnits) ? 1 : m->conversion();
-            metricValue += (rideNavigator->context->athlete->useMetricUnits) ? 0 : m->conversionSum();
-
-            // format with the right precision
-            if (m->units(true) == "seconds" || m->units(true) == tr("seconds")) {
-                value = QTime(0,0,0,0).addSecs(metricValue).toString("hh:mm:ss");
-            } else {
-                value = QString("%1").arg(metricValue, 0, 'f', m->precision());
-            }
-
-        } else {
-
-            // blank out zero values, they look ugly and are distracting
-            value = "";
-        }
+        // get rid of 0 its ugly
+        if (value =="nan" || value == "0" || value == "0.0" || value == "0.00") value="";
 
     } else {
         // is this the ride date/time ?
@@ -1025,14 +1134,24 @@ void NavigatorCellDelegate::paint(QPainter *painter, const QStyleOptionViewItem
     }
 
     // normal render
+    bool isnormal=false;
     QString calendarText = rideNavigator->tableView->model()->data(index, Qt::UserRole).toString();
     QColor userColor = rideNavigator->tableView->model()->data(index, Qt::BackgroundRole).value<QBrush>().color();
     if (userColor == QColor(1,1,1)) {
         rideBG = false; // default so don't swap round...
+        isnormal = true; // just default so no bg or box
         userColor = GColor(CPLOTMARKER);
     }
+
+    // basic background
     QBrush background = QBrush(GColor(CPLOTBACKGROUND));
 
+    // runs are darker
+    //if (isRun) {
+        //background.setColor(background.color().darker(150));
+        //userColor = userColor.darker(150);
+    //}
+
     if (columnName != "*") {
 
         myOption.displayAlignment = Qt::AlignLeft | Qt::AlignTop;
@@ -1067,6 +1186,8 @@ void NavigatorCellDelegate::paint(QPainter *painter, const QStyleOptionViewItem
             // not selected, so invert ride plot color
             if (hover) painter->setPen(QColor(Qt::black));
             else painter->setPen(rideBG ? rideNavigator->reverseColor : userColor);
+        } else if (!focus) { // selected but out of focus //
+            painter->setPen(QColor(Qt::black));
         }
 
         QRect normal(myOption.rect.x(), myOption.rect.y()+1, myOption.rect.width(), myOption.rect.height());
@@ -1092,7 +1213,7 @@ void NavigatorCellDelegate::paint(QPainter *painter, const QStyleOptionViewItem
             myOption.font.setWeight(QFont::Normal);
 
             if (hover) painter->fillRect(myOption.rect, QColor(Qt::lightGray)); 
-            else painter->fillRect(myOption.rect, rideBG ? userColor : GColor(CPLOTBACKGROUND));
+            else painter->fillRect(myOption.rect, rideBG ? userColor : background.color());
 
             drawDisplay(painter, myOption, myOption.rect, "");
             myOption.rect.setX(10); // wider notes display
@@ -1108,9 +1229,9 @@ void NavigatorCellDelegate::paint(QPainter *painter, const QStyleOptionViewItem
             painter->setPen(isColor);
 
 #if (defined (Q_OS_MAC) && (QT_VERSION >= 0x050000)) // on QT5 the scrollbars have no width
-            if (!selected && !rideBG && high.x()+12 > rideNavigator->geometry().width() && userColor != GColor(CPLOTMARKER)) {
+            if (!selected && !rideBG && high.x()+12 > rideNavigator->geometry().width() && !isnormal) {
 #else
-            if (!selected && !rideBG && high.x()+32 > rideNavigator->geometry().width() && userColor != GColor(CPLOTMARKER)) {
+            if (!selected && !rideBG && high.x()+32 > rideNavigator->geometry().width() && !isnormal) {
 #endif
                 painter->fillRect(high, userColor);
             } else {
@@ -1144,6 +1265,11 @@ void NavigatorCellDelegate::paint(QPainter *painter, const QStyleOptionViewItem
     }
 }
 
+static bool insensitiveLessThan(const QString &a, const QString &b)
+{
+    return a.toLower() < b.toLower();
+}
+
 ColumnChooser::ColumnChooser(QList<QString>&logicalHeadings)
 {
     // wipe away everything when you close please...
@@ -1154,7 +1280,15 @@ ColumnChooser::ColumnChooser(QList<QString>&logicalHeadings)
     clicked = new QSignalMapper(this); // maps each button click event
     connect(clicked, SIGNAL(mapped(const QString &)), this, SLOT(buttonClicked(const QString &)));
 
-    buttons = new QGridLayout(this);
+    QVBoxLayout *us = new QVBoxLayout(this);
+    us->setSpacing(0);
+    us->setContentsMargins(0,0,0,0);
+    
+    scrollarea = new QScrollArea(this);
+    us->addWidget(scrollarea);
+
+    QWidget *but = new QWidget(this);
+    buttons = new QVBoxLayout(but);
     buttons->setSpacing(0);
     buttons->setContentsMargins(0,0,0,0);
 
@@ -1162,30 +1296,35 @@ ColumnChooser::ColumnChooser(QList<QString>&logicalHeadings)
     small.setPointSize(8);
 
     QList<QString> buttonNames = logicalHeadings;
-    qSort(buttonNames);
+    qSort(buttonNames.begin(), buttonNames.end(), insensitiveLessThan);
 
-    int x = 0;
-    int y = 0;
+    QString last;
     foreach (QString column, buttonNames) {
 
+        // ignore groupby
         if (column == "*") continue;
 
+        // ignore meta fields that are metrics or duplicates
+        if (column == last || column.contains("_")) continue;
+
         // setup button
         QPushButton *add = new QPushButton(column, this);
         add->setFont(small);
         add->setContentsMargins(0,0,0,0);
-        buttons->addWidget(add, y, x);
+        buttons->addWidget(add);
 
         connect(add, SIGNAL(pressed()), clicked, SLOT(map()));
         clicked->setMapping(add, column);
 
-        // update layout
-        x++;
-        if (x > 5) {
-            y++;
-            x = 0;
-        }
+        // for spotting duplicates
+        last = column;
+
     }
+    scrollarea->setWidget(but);
+
+    but->setFixedWidth(230);
+    scrollarea->setFixedWidth(250);
+    setFixedWidth(250);
 }
 
 void
