@@ -45,6 +45,8 @@ class DataFilter;
 class GcMiniCalendar;
 class SearchBox;
 class RideTreeView;
+class EditMetricDetailDialog;
+class EditUserDataDialog;
 
 //
 // The RideNavigator
@@ -71,6 +73,8 @@ class RideNavigator : public GcWindow
     friend class ::GcMiniCalendar;
     friend class ::DataFilter;
     friend class ::SearchBox;
+    friend class ::EditMetricDetailDialog;
+    friend class ::EditUserDataDialog;
 
     public:
         RideNavigator(Context *, bool mainwindow = false);
