@@ -54,6 +54,8 @@ public:
     virtual bool wasUploaded();
     virtual void upload()=0;
 
+    QString insertedName;
+
 private:
     QString _name;
 
@@ -61,7 +63,6 @@ protected:
     Context *context;
     RideItem *ride;
     ShareDialog *parent;
-
 };
 
 // uploader to strava.com
@@ -262,6 +263,7 @@ private:
      QCheckBox *selfLoopsChk;
      QCheckBox *veloHeroChk;
      QCheckBox *trainingstagebuchChk;
+     QCheckBox *sportplushealthChk;
      //QCheckBox *garminChk;
 
      RideItem *ride;
@@ -272,6 +274,7 @@ private:
      ShareDialogUploader *selfLoopsUploader;
      ShareDialogUploader *veloHeroUploader;
      ShareDialogUploader *trainingstagebuchUploader;
+     ShareDialogUploader *sportplushealthUploader;
      //ShareDialogUploader *garminUploader;
 
      QString athleteId;
