@@ -20,7 +20,7 @@
 #include "RideMetric.h"
 #include "BestIntervalDialog.h"
 #include "Zones.h"
-#include <math.h>
+#include <cmath>
 #include <QApplication>
 
 class ZoneTime : public RideMetric {
@@ -41,6 +41,7 @@ class ZoneTime : public RideMetric {
         setPrecision(0);
         setConversion(1.0);
     }
+    bool isTime() const { return true; }
     void setLevel(int level) { this->level=level-1; } // zones start from zero not 1
     void compute(const RideFile *ride, const Zones *zone, int zoneRange,
                  const HrZones *, int,
@@ -49,7 +50,7 @@ class ZoneTime : public RideMetric {
     {
         seconds = 0;
         // get zone ranges
-        if (zone && zoneRange >= 0) {
+        if (zone && zoneRange >= 0 && ride->areDataPresent()->watts) {
             // iterate and compute
             foreach(const RideFilePoint *point, ride->dataPoints()) {
                 if (zone->whichZone(zoneRange, point->watts) == level)
