@@ -36,6 +36,9 @@
 // Three current realtime device types supported are:
 #include "RealtimeController.h"
 #include "ComputrainerController.h"
+#if QT_VERSION >= 0x050000
+#include "MonarkController.h"
+#endif
 #include "ANTlocalController.h"
 #include "NullController.h"
 #ifdef GC_HAVE_WFAPI
@@ -113,6 +116,48 @@ TrainSidebar::TrainSidebar(Context *context) : GcWindow(context), context(contex
     QStyle *cde = QStyleFactory::create(OS_STYLE);
     mediaTree->verticalScrollBar()->setStyle(cde);
 #endif
+
+
+
+    videosyncModel = new QSqlTableModel(this, trainDB->connection());
+    videosyncModel->setTable("videosyncs");
+    videosyncModel->setEditStrategy(QSqlTableModel::OnManualSubmit);
+    videosyncModel->select();
+    while (videosyncModel->canFetchMore(QModelIndex())) videosyncModel->fetchMore(QModelIndex());
+
+    vssortModel = new QSortFilterProxyModel(this);
+    vssortModel->setSourceModel(videosyncModel);
+    vssortModel->setDynamicSortFilter(true);
+    vssortModel->sort(1, Qt::AscendingOrder); //filename order
+
+    videosyncTree = new QTreeView;
+    videosyncTree->setModel(vssortModel);
+
+    // hide unwanted columns and header
+    for(int i=0; i<videosyncTree->header()->count(); i++)
+        videosyncTree->setColumnHidden(i, true);
+    videosyncTree->setColumnHidden(1, false); // show filename
+    videosyncTree->header()->hide();
+
+    videosyncTree->setSortingEnabled(false);
+    videosyncTree->setAlternatingRowColors(false);
+    videosyncTree->setEditTriggers(QAbstractItemView::NoEditTriggers); // read-only
+    videosyncTree->expandAll();
+    videosyncTree->header()->setCascadingSectionResizes(true); // easier to resize this way
+    videosyncTree->setContextMenuPolicy(Qt::CustomContextMenu);
+    videosyncTree->header()->setStretchLastSection(true);
+    videosyncTree->header()->setMinimumSectionSize(0);
+    videosyncTree->header()->setFocusPolicy(Qt::NoFocus);
+    videosyncTree->setFrameStyle(QFrame::NoFrame);
+#ifdef Q_OS_MAC
+    videosyncTree->header()->setSortIndicatorShown(false); // blue looks nasty
+    videosyncTree->setAttribute(Qt::WA_MacShowFocusRect, 0);
+#endif
+#ifdef Q_OS_WIN
+    QStyle *cdevideosync = QStyleFactory::create(OS_STYLE);
+    videosyncTree->verticalScrollBar()->setStyle(cdevideosync);
+#endif
+
 #endif
 
     deviceTree = new QTreeWidget;
@@ -154,6 +199,7 @@ TrainSidebar::TrainSidebar(Context *context) : GcWindow(context), context(contex
     workoutTree->setFrameStyle(QFrame::NoFrame);
     workoutTree->setAlternatingRowColors(false);
     workoutTree->setEditTriggers(QAbstractItemView::NoEditTriggers); // read-only
+    workoutTree->setSelectionMode(QAbstractItemView::ExtendedSelection);
     workoutTree->expandAll();
     workoutTree->header()->setCascadingSectionResizes(true); // easier to resize this way
     workoutTree->setContextMenuPolicy(Qt::CustomContextMenu);
@@ -191,6 +237,9 @@ TrainSidebar::TrainSidebar(Context *context) : GcWindow(context), context(contex
     rewind->setStyleSheet("background-color: rgba( 255, 255, 255, 0% ); border: 0px;");
     rewind->setAutoRepeat(true);
     rewind->setAutoRepeatDelay(200);
+#if QT_VERSION > 0x050400
+    rewind->setShortcut(Qt::Key_MediaPrevious);
+#endif
     toolbuttons->addWidget(rewind);
 
     QIcon stopIcon(":images/oxygen/stop.png");
@@ -201,6 +250,9 @@ TrainSidebar::TrainSidebar(Context *context) : GcWindow(context), context(contex
     stop->setAutoDefault(false);
     stop->setFlat(true);
     stop->setStyleSheet("background-color: rgba( 255, 255, 255, 0% ); border: 0px;");
+#if QT_VERSION > 0x050400
+    stop->setShortcut(Qt::Key_MediaStop);
+#endif
     toolbuttons->addWidget(stop);
 
     QIcon playIcon(":images/oxygen/play.png");
@@ -211,6 +263,7 @@ TrainSidebar::TrainSidebar(Context *context) : GcWindow(context), context(contex
     play->setAutoDefault(false);
     play->setFlat(true);
     play->setStyleSheet("background-color: rgba( 255, 255, 255, 0% ); border: 0px;");
+    play->setShortcut(Qt::Key_MediaTogglePlayPause);
     toolbuttons->addWidget(play);
 
     QIcon fwdIcon(":images/oxygen/ffwd.png");
@@ -223,6 +276,9 @@ TrainSidebar::TrainSidebar(Context *context) : GcWindow(context), context(contex
     forward->setStyleSheet("background-color: rgba( 255, 255, 255, 0% ); border: 0px;");
     forward->setAutoRepeat(true);
     forward->setAutoRepeatDelay(200);
+#if QT_VERSION > 0x050400
+    forward->setShortcut(Qt::Key_MediaNext);
+#endif
     toolbuttons->addWidget(forward);
 
     QIcon lapIcon(":images/oxygen/lap.png");
@@ -233,6 +289,9 @@ TrainSidebar::TrainSidebar(Context *context) : GcWindow(context), context(contex
     lap->setAutoDefault(false);
     lap->setFlat(true);
     lap->setStyleSheet("background-color: rgba( 255, 255, 255, 0% ); border: 0px;");
+#if QT_VERSION > 0x050400
+    lap->setShortcut(Qt::Key_0);
+#endif
     toolbuttons->addWidget(lap);
     toolbuttons->addStretch();
 
@@ -334,6 +393,13 @@ intensity->hide(); //XXX!!! temporary
     connect(moreMediaAct, SIGNAL(triggered(void)), this, SLOT(mediaPopup(void)));
     mediaItem->addWidget(mediaTree);
     trainSplitter->addWidget(mediaItem);
+
+    videosyncItem = new GcSplitterItem(tr("VideoSync"), iconFromPNG(":images/sidebar/sync.png"), this);
+    QAction *moreVideoSyncAct = new QAction(iconFromPNG(":images/sidebar/extra.png"), tr("Menu"), this);
+    videosyncItem->addAction(moreVideoSyncAct);
+    connect(moreVideoSyncAct, SIGNAL(triggered(void)), this, SLOT(videosyncPopup(void)));
+    videosyncItem->addWidget(videosyncTree);
+    trainSplitter->addWidget(videosyncItem);
 #endif
     trainSplitter->prepare(context->athlete->cyclist, "train");
 
@@ -341,6 +407,7 @@ intensity->hide(); //XXX!!! temporary
     // get rid of annoying focus rectangle for sidebar components
 #if !defined GC_VIDEO_NONE
     mediaTree->setAttribute(Qt::WA_MacShowFocusRect, 0);
+    videosyncTree->setAttribute(Qt::WA_MacShowFocusRect, 0);
 #endif
     workoutTree->setAttribute(Qt::WA_MacShowFocusRect, 0);
     deviceTree->setAttribute(Qt::WA_MacShowFocusRect, 0);
@@ -354,18 +421,19 @@ intensity->hide(); //XXX!!! temporary
     connect(mediaTree->selectionModel(), SIGNAL(selectionChanged(QItemSelection, QItemSelection)),
                             this, SLOT(mediaTreeWidgetSelectionChanged()));
     connect(context, SIGNAL(selectMedia(QString)), this, SLOT(selectVideo(QString)));
+    connect(videosyncTree->selectionModel(), SIGNAL(selectionChanged(QItemSelection, QItemSelection)),
+                            this, SLOT(videosyncTreeWidgetSelectionChanged()));
+    connect(context, SIGNAL(selectVideoSync(QString)), this, SLOT(selectVideoSync(QString)));
 #endif
     connect(context, SIGNAL(configChanged(qint32)), this, SLOT(configChanged(qint32)));
     connect(context, SIGNAL(selectWorkout(QString)), this, SLOT(selectWorkout(QString)));
     connect(trainDB, SIGNAL(dataChanged()), this, SLOT(refresh()));
 
     connect(workoutTree->selectionModel(), SIGNAL(selectionChanged(QItemSelection, QItemSelection)), this, SLOT(workoutTreeWidgetSelectionChanged()));
-    // add a watch on all directories
-    QVariant workoutDir = appsettings->value(NULL, GC_WORKOUTDIR);
 
-    // set home
     //XXX ??? main = parent;
     ergFile = NULL;
+    videosyncFile = NULL;
     calibrating = false;
 
     // now the GUI is setup lets sort our control variables
@@ -424,6 +492,17 @@ TrainSidebar::refresh()
 
     // restore selection
     selectVideo(videoPath);
+
+    // remember selection
+    row = videosyncTree->currentIndex().row();
+    QString videosyncPath = videosyncTree->model()->data(videosyncTree->model()->index(row,0)).toString();
+
+    // refresh data
+    videosyncModel->select();
+    while (videosyncModel->canFetchMore(QModelIndex())) videosyncModel->fetchMore(QModelIndex());
+
+    // restore selection
+    selectVideoSync(videosyncPath);
 #endif
 
     row = workoutTree->currentIndex().row();
@@ -451,12 +530,27 @@ TrainSidebar::workoutPopup()
     menu.addAction(wizard);
     menu.addAction(scan);
 
+
     // we can delete too
-    QModelIndex current = workoutTree->currentIndex();
-    QModelIndex target = sortModel->mapToSource(current);
-    QString filename = workoutModel->data(workoutModel->index(target.row(), 0), Qt::DisplayRole).toString();
-    if (QFileInfo(filename).exists()) {
-        QAction *del = new QAction(tr("Delete selected workout"), workoutTree);
+    int delNumber = 0;
+
+    QModelIndexList list = workoutTree->selectionModel()->selectedRows();
+    foreach (QModelIndex index, list)
+    {
+        QModelIndex target = sortModel->mapToSource(index);
+
+        QString filename = workoutModel->data(workoutModel->index(target.row(), 0), Qt::DisplayRole).toString();
+        if (QFileInfo(filename).exists()) {
+            delNumber++;
+        }
+    }
+
+    if (delNumber > 0) {
+        QAction *del = new QAction(tr("Delete selected Workout"), workoutTree);
+
+        if (delNumber > 1) {
+            del->setText(QString(tr("Delete %1 selected Workouts")).arg(delNumber));
+        }
         menu.addAction(del);
         connect(del, SIGNAL(triggered(void)), this, SLOT(deleteWorkouts(void)));
     }
@@ -491,7 +585,7 @@ TrainSidebar::mediaPopup()
     QModelIndex target = vsortModel->mapToSource(current);
     QString filename = videoModel->data(videoModel->index(target.row(), 0), Qt::DisplayRole).toString();
     if (QFileInfo(filename).exists()) {
-        QAction *del = new QAction(tr("Delete selected video"), workoutTree);
+        QAction *del = new QAction(tr("Remove reference to selected video"), workoutTree);
         menu.addAction(del);
         connect(del, SIGNAL(triggered(void)), this, SLOT(deleteVideos(void)));
     }
@@ -501,12 +595,42 @@ TrainSidebar::mediaPopup()
                                            mediaItem->pos().y())));
 }
 
+void
+TrainSidebar::videosyncPopup()
+{
+    // OK - we are working with a specific event..
+    QMenu menu(videosyncTree);
+    QAction *import = new QAction(tr("Import VideoSync from File"), videosyncTree);
+    QAction *scan = new QAction(tr("Scan for VideoSyncs"), videosyncTree);
+
+    menu.addAction(import);
+    menu.addAction(scan);
+
+    // connect menu to functions
+    connect(import, SIGNAL(triggered(void)), context->mainWindow, SLOT(importWorkout(void)));
+    connect(scan, SIGNAL(triggered(void)), context->mainWindow, SLOT(manageLibrary(void)));
+
+    QModelIndex current = videosyncTree->currentIndex();
+    QModelIndex target = vssortModel->mapToSource(current);
+    QString filename = videosyncModel->data(videosyncModel->index(target.row(), 0), Qt::DisplayRole).toString();
+    if (QFileInfo(filename).exists()) {
+        QAction *del = new QAction(tr("Delete selected VideoSync"), workoutTree);
+        menu.addAction(del);
+        connect(del, SIGNAL(triggered(void)), this, SLOT(deleteVideoSyncs(void)));
+    }
+
+    // execute the menu
+    menu.exec(trainSplitter->mapToGlobal(QPoint(videosyncItem->pos().x()+videosyncItem->width()-20,
+                                           videosyncItem->pos().y())));
+}
+
 void
 TrainSidebar::configChanged(qint32)
 {
     setProperty("color", GColor(CTRAINPLOTBACKGROUND));
 #if !defined GC_VIDEO_NONE
     mediaTree->setStyleSheet(GCColor::stylesheet());
+    videosyncTree->setStyleSheet(GCColor::stylesheet());
 #endif
     workoutTree->setStyleSheet(GCColor::stylesheet());
     deviceTree->setStyleSheet(GCColor::stylesheet());
@@ -539,6 +663,10 @@ TrainSidebar::configChanged(qint32)
         // to interact with the device
         if (Devices.at(i).type == DEV_CT) {
             Devices[i].controller = new ComputrainerController(this, &Devices[i]);
+#if QT_VERSION >= 0x050000
+        } else if (Devices.at(i).type == DEV_MONARK) {
+            Devices[i].controller = new MonarkController(this, &Devices[i]);
+#endif
 #ifdef GC_HAVE_LIBUSB
         } else if (Devices.at(i).type == DEV_FORTIUS) {
             Devices[i].controller = new FortiusController(this, &Devices[i]);
@@ -662,6 +790,7 @@ TrainSidebar::workoutTreeWidgetSelectionChanged()
             delete ergFile;
             ergFile = NULL;
             context->notifyErgFileSelected(NULL);
+            removeInvalidWorkout();
             mode = ERG;
             status &= ~RT_WORKOUT;
             setLabels();
@@ -707,9 +836,9 @@ TrainSidebar::deleteVideos()
     if (QFileInfo(filename).exists()) {
         // are you sure?
         QMessageBox msgBox;
-        msgBox.setText(tr("Are you sure you want to delete this video?"));
+        msgBox.setText(tr("Are you sure you want to remove the reference to this video?"));
         msgBox.setInformativeText(filename);
-        QPushButton *deleteButton = msgBox.addButton(tr("Delete"),QMessageBox::YesRole);
+        QPushButton *deleteButton = msgBox.addButton(tr("Remove"),QMessageBox::YesRole);
         msgBox.setStandardButtons(QMessageBox::Cancel);
         msgBox.setDefaultButton(QMessageBox::Cancel);
         msgBox.setIcon(QMessageBox::Critical);
@@ -730,17 +859,19 @@ TrainSidebar::deleteVideos()
         trainDB->endLUW();
     }
 }
+
+
 void
-TrainSidebar::deleteWorkouts()
+TrainSidebar::deleteVideoSyncs()
 {
-    QModelIndex current = workoutTree->currentIndex();
-    QModelIndex target = sortModel->mapToSource(current);
-    QString filename = workoutModel->data(workoutModel->index(target.row(), 0), Qt::DisplayRole).toString();
+    QModelIndex current = videosyncTree->currentIndex();
+    QModelIndex target = vssortModel->mapToSource(current);
+    QString filename = videosyncModel->data(videosyncModel->index(target.row(), 0), Qt::DisplayRole).toString();
 
     if (QFileInfo(filename).exists()) {
         // are you sure?
         QMessageBox msgBox;
-        msgBox.setText(tr("Are you sure you want to delete this workout?"));
+        msgBox.setText(tr("Are you sure you want to delete this VideoSync?"));
         msgBox.setInformativeText(filename);
         QPushButton *deleteButton = msgBox.addButton(tr("Delete"),QMessageBox::YesRole);
         msgBox.setStandardButtons(QMessageBox::Cancel);
@@ -752,13 +883,114 @@ TrainSidebar::deleteWorkouts()
 
         // delete from disk
         QFile(filename).remove();
+
         // delete from DB
         trainDB->startLUW();
-        trainDB->deleteWorkout(filename);
+        trainDB->deleteVideoSync(filename);
         trainDB->endLUW();
     }
 }
 
+void
+TrainSidebar::removeInvalidVideoSync()
+{
+    QModelIndex current = videosyncTree->currentIndex();
+    QModelIndex target = vssortModel->mapToSource(current);
+    QString filename = videosyncModel->data(videosyncModel->index(target.row(), 0), Qt::DisplayRole).toString();
+    QMessageBox msgBox;
+    msgBox.setText(tr("The VideoSync file is either not valid or not existing and will be removed from the library."));
+    msgBox.setInformativeText(filename);
+    QPushButton *removeButton = msgBox.addButton(tr("Remove"),QMessageBox::YesRole);
+    msgBox.setStandardButtons(QMessageBox::Cancel);
+    msgBox.setDefaultButton(removeButton);
+    msgBox.setIcon(QMessageBox::Critical);
+    msgBox.exec();
+
+    if(msgBox.clickedButton() != removeButton) return;
+
+    // delete from DB
+    trainDB->startLUW();
+    trainDB->deleteVideoSync(filename);
+    trainDB->endLUW();
+
+}
+
+void
+TrainSidebar::deleteWorkouts()
+{
+    QStringList nameList;
+    QModelIndexList list = workoutTree->selectionModel()->selectedRows();
+    foreach (QModelIndex index, list) {
+        QModelIndex target = sortModel->mapToSource(index);
+        QString filename = workoutModel->data(workoutModel->index(target.row(), 0), Qt::DisplayRole).toString();
+        if (QFileInfo(filename).exists()) {
+            nameList.append(filename);
+        }
+    }
+
+    if (nameList.count()>0) {
+        // are you sure?
+        QMessageBox msgBox;
+        if (nameList.count()==1) {
+            msgBox.setText(tr("Are you sure you want to delete this Workout?"));
+        }
+        else {
+            msgBox.setText(QString(tr("Are you sure you want to delete this %1 workouts?")).arg(nameList.count()));
+        }
+        QString info;
+        for (int i=0;i<nameList.count() && i<21;i++) {
+           info += nameList.at(i)+"\n";
+           if (i == 20) {
+               info += "...\n";
+           }
+        }
+        msgBox.setInformativeText(info);
+        QPushButton *deleteButton = msgBox.addButton(tr("Delete"),QMessageBox::YesRole);
+        msgBox.setStandardButtons(QMessageBox::Cancel);
+        msgBox.setDefaultButton(QMessageBox::Cancel);
+        msgBox.setIcon(QMessageBox::Critical);
+        msgBox.exec();
+
+        if(msgBox.clickedButton() != deleteButton) return;
+
+        foreach (QString filename, nameList) {
+            if (QFileInfo(filename).exists()) {
+                // delete from disk
+                QFile(filename).remove();
+                // delete from DB
+                trainDB->startLUW();
+                trainDB->deleteWorkout(filename);
+                trainDB->endLUW();
+            }
+        }
+    }
+}
+
+void
+TrainSidebar::removeInvalidWorkout()
+{
+    QModelIndex current = workoutTree->currentIndex();
+    QModelIndex target = sortModel->mapToSource(current);
+    QString filename = workoutModel->data(workoutModel->index(target.row(), 0), Qt::DisplayRole).toString();
+
+    QMessageBox msgBox;
+    msgBox.setText(tr("The Workout file is either not valid or not existing and will be removed from the library."));
+    msgBox.setInformativeText(filename);
+    QPushButton *removeButton = msgBox.addButton(tr("Remove"),QMessageBox::YesRole);
+    msgBox.setStandardButtons(QMessageBox::Cancel);
+    msgBox.setDefaultButton(removeButton);
+    msgBox.setIcon(QMessageBox::Critical);
+    msgBox.exec();
+
+    if(msgBox.clickedButton() != removeButton) return;
+
+    // delete from DB
+    trainDB->startLUW();
+    trainDB->deleteWorkout(filename);
+    trainDB->endLUW();
+
+}
+
 void
 TrainSidebar::mediaTreeWidgetSelectionChanged()
 {
@@ -769,6 +1001,43 @@ TrainSidebar::mediaTreeWidgetSelectionChanged()
     context->notifyMediaSelected(filename);
 }
 
+void
+TrainSidebar::videosyncTreeWidgetSelectionChanged()
+{
+
+    QModelIndex current = videosyncTree->currentIndex();
+    QModelIndex target = vssortModel->mapToSource(current);
+    QString filename = videosyncModel->data(videosyncModel->index(target.row(), 0), Qt::DisplayRole).toString();
+
+    // wip away the current selected videosync
+    if (videosyncFile) {
+        delete videosyncFile;
+        videosyncFile = NULL;
+    }
+
+     if (filename == "") {
+        context->notifyVideoSyncFileSelected(NULL);
+        return;
+    }
+
+    // is "None" selected?
+    int index = target.row();
+    if (index == 0) {
+        // None menu entry
+        context->notifyVideoSyncFileSelected(NULL);
+    } else {
+        videosyncFile = new VideoSyncFile(filename, mode, context);
+        if (videosyncFile->isValid()) {
+            context->notifyVideoSyncFileSelected(videosyncFile);
+        } else {
+            delete videosyncFile;
+            videosyncFile = NULL;
+            context->notifyVideoSyncFileSelected(NULL);
+            removeInvalidVideoSync();
+        }
+    }
+}
+
 /*--------------------------------------------------------------------------------
  * Was realtime window, now local and manages controller and chart updates etc
  *------------------------------------------------------------------------------*/
@@ -794,6 +1063,7 @@ void TrainSidebar::Start()       // when start button is pressed
 
 #if !defined GC_VIDEO_NONE
         mediaTree->setEnabled(false);
+        videosyncTree->setEnabled(false);
 #endif
 
         // tell the world
@@ -816,6 +1086,7 @@ void TrainSidebar::Start()       // when start button is pressed
 #if !defined GC_VIDEO_NONE
         // enable media tree so we can change movie - mid workout
         mediaTree->setEnabled(true);
+        videosyncTree->setEnabled(true);
 #endif
 
         // tell the world
@@ -842,6 +1113,7 @@ void TrainSidebar::Start()       // when start button is pressed
 
 #if !defined GC_VIDEO_NONE
         mediaTree->setEnabled(false);
+        videosyncTree->setEnabled(false);
 #endif
         workoutTree->setEnabled(false);
         deviceTree->setEnabled(false);
@@ -849,7 +1121,7 @@ void TrainSidebar::Start()       // when start button is pressed
         // START!
         play->setIcon(pauseIcon);
 
-        load = 0;
+        load = 100;
         slope = 0.0;
 
         if (mode == ERG || mode == MRC) {
@@ -935,6 +1207,7 @@ void TrainSidebar::Pause()        // pause capture to recalibrate
 
 #if !defined GC_VIDEO_NONE
         mediaTree->setEnabled(false);
+        videosyncTree->setEnabled(false);
 #endif
 
         // tell the world
@@ -954,6 +1227,7 @@ void TrainSidebar::Pause()        // pause capture to recalibrate
         // enable media tree so we can change movie
 #if !defined GC_VIDEO_NONE
         mediaTree->setEnabled(true);
+        videosyncTree->setEnabled(true);
 #endif
 
         // tell the world
@@ -971,6 +1245,7 @@ void TrainSidebar::Stop(int deviceStatus)        // when stop button is pressed
     // media or workouts whilst a workout is in progress
 #if !defined GC_VIDEO_NONE
     mediaTree->setEnabled(true);
+    videosyncTree->setEnabled(true);
 #endif
     workoutTree->setEnabled(true);
     deviceTree->setEnabled(true);
@@ -1114,14 +1389,14 @@ void TrainSidebar::guiUpdate()           // refreshes the telemetry
 
 				if (Devices[dev].type == DEV_FORTIUS) {
 	                rtData.setLoad(local.getLoad()); // and get load in case it was adjusted
-                    rtData.setSlope(local.getSlope()); // and get slope in case it was adjusted	
-					// to within defined limits					
+                    rtData.setSlope(local.getSlope()); // and get slope in case it was adjusted
+					// to within defined limits
 				}
 
                 if (Devices[dev].type == DEV_ANTLOCAL || Devices[dev].type == DEV_NULL) {
                     rtData.setHb(local.getSmO2(), local.gettHb()); //only moxy data from ant and robot devices right now
                 }
-				
+
                 // what are we getting from this one?
                 if (dev == bpmTelemetry) rtData.setHr(local.getHr());
                 if (dev == rpmTelemetry) rtData.setCadence(local.getCadence());
@@ -1138,11 +1413,27 @@ void TrainSidebar::guiUpdate()           // refreshes the telemetry
                     rtData.setLPS(local.getLPS());
                     rtData.setRPS(local.getRPS());
                 }
+                if (local.getTrainerStatusAvailable())
+                {
+                    rtData.setTrainerStatusAvailable(true);
+                    rtData.setTrainerReady(local.getTrainerReady());
+                    rtData.setTrainerRunning(local.getTrainerRunning());
+                    rtData.setTrainerCalibRequired(local.getTrainerCalibRequired());
+                    rtData.setTrainerConfigRequired(local.getTrainerConfigRequired());
+                    rtData.setTrainerBrakeFault(local.getTrainerBrakeFault());
+                }
             }
 
             // Distance assumes current speed for the last second. from km/h to km/sec
             displayDistance += displaySpeed / (5 * 3600); // assumes 200ms refreshrate
-            displayWorkoutDistance += displaySpeed / (5 * 3600); // assumes 200ms refreshrate
+
+            if (!(status&RT_MODE_ERGO) && (context->currentVideoSyncFile()))
+            {
+                displayWorkoutDistance = context->currentVideoSyncFile()->km + context->currentVideoSyncFile()->manualOffset;
+                // TODO : graphs to be shown at seek position
+            }
+            else
+                displayWorkoutDistance += displaySpeed / (5 * 3600); // assumes 200ms refreshrate
             rtData.setDistance(displayDistance);
 
             // time
@@ -1435,8 +1726,11 @@ void TrainSidebar::FFwd()
         load_msecs += 10000; // jump forward 10 seconds
         context->notifySeek(load_msecs);
     }
+    else if (context->currentVideoSyncFile())
+    {
+        context->notifySeek(+1); // in case of video with RLV file synchronisation just ask to go forward
+    }
     else displayWorkoutDistance += 1; // jump forward a kilometer in the workout
-
 }
 
 void TrainSidebar::Rewind()
@@ -1447,7 +1741,12 @@ void TrainSidebar::Rewind()
         load_msecs -=10000; // jump back 10 seconds
         if (load_msecs < 0) load_msecs = 0;
         context->notifySeek(load_msecs);
-    } else {
+    }
+    else if (context->currentVideoSyncFile())
+    {
+        context->notifySeek(-1); // in case of video with RLV file synchronisation just ask to go backward
+    }
+    else {
         displayWorkoutDistance -=1; // jump back a kilometer
         if (displayWorkoutDistance < 0) displayWorkoutDistance = 0;
     }
@@ -1784,6 +2083,19 @@ TrainSidebar::selectVideo(QString fullpath)
     }
 }
 
+void
+TrainSidebar::selectVideoSync(QString fullpath)
+{
+    // look at each entry in the top videosyncTree
+    for (int i=0; i<videosyncTree->model()->rowCount(); i++) {
+        QString path = videosyncTree->model()->data(videosyncTree->model()->index(i,0)).toString();
+        if (path == fullpath) {
+            videosyncTree->setCurrentIndex(videosyncTree->model()->index(i,0));
+            break;
+        }
+    }
+}
+
 // we have been told to select this workout (usually because
 // the user just dragndropped it in)
 void
