@@ -22,7 +22,7 @@
 #include <QFile>
 #include <QTextStream>
 #include <algorithm> // for std::sort
-#include "math.h"
+#include "cmath"
 
 static int wkoFileReaderRegistered = RideFileFactory::instance().registerReader(
                                      "wko", "WKO+ Files", new WkoFileReader());
@@ -179,7 +179,7 @@ WkoParser::WkoParser(QFile &file, QStringList &errors, QList<RideFile*>*rides)
             add.stop = datapoints.at(references.at(i)->stop)->secs + results->recIntSecs()-.001;
         else continue;
 
-        results->addInterval(add.start, add.stop, add.name);
+        results->addInterval(RideFileInterval::DEVICE, add.start, add.stop, add.name);
     }
 
     // free up temporary storage for range post processing
@@ -210,7 +210,7 @@ WkoParser::parseRawData(WKO_UCHAR *fb)
     double cad=0, hr=0, km=0, kph=0, nm=0, watts=0, slope=0, alt=0, lon=0, lat=0, wind=0, temp=RideFile::NoTemp, interval=0;
 
     int isnull=0;
-    WKO_ULONG records, data;
+    WKO_ULONG records=0, data;
     WKO_UCHAR *thelot;
     WKO_USHORT us;
 
@@ -489,7 +489,10 @@ WkoParser::parseRawData(WKO_UCHAR *fb)
                     results->appendPoint((double)rtime/1000, cad, hr, km,
                             kph, nm, watts, alt, lon, lat, wind, slope, temp, 0.0, 
                             0.0,0.0,0.0,0.0, // vector pedal torque eff and smoothness not supported in WKO (?)
+                            0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                             0.0,0.0,
+                            0.0,0.0,0.0, // running dynamics
+                            0.0, // tcore
                             0);
             }
 
@@ -804,6 +807,14 @@ WkoParser::parseHeaderData(WKO_UCHAR *fb)
     num=0;
 
     while (charts) {     // keep parsing until we have no charts left
+
+        // basic bounds check
+        if ((p - fb) > bufferSize) {
+            errors << "Buffer overrun, file may be corrupt / truncated";
+            qDebug()<<"buffer overrun";
+            return NULL;
+        }
+
         enum configtype type=INVALID;
 
         p += donumber(p, &ul);
