@@ -116,6 +116,7 @@ bool Zones::read(QFile &file)
                 "\\s*:?\\s*$",                                       // optional :
                 Qt::CaseInsensitive)
     };
+    QRegExp ftpx("^FTP=(\\d+)$");
     QRegExp wprimerx("^W'=(\\d+)$");
     QRegExp pmaxx("^Pmax=(\\d+)$");
     QRegExp zonerx("^\\s*([^ ,][^,]*),\\s*([^ ,][^,]*),\\s*"
@@ -131,6 +132,7 @@ bool Zones::read(QFile &file)
     bool in_range = false;
     QDate begin = date_zero, end = date_infinity;
     int cp=0;
+    int ftp=0;
     int wprime=0;
     int pmax=0;
     QList<ZoneInfo> zoneInfos;
@@ -172,7 +174,7 @@ bool Zones::read(QFile &file)
                 if (in_range) {
 
                     // if zones are empty, then generate them
-                    ZoneRange range(begin, end, cp, wprime ? wprime : defaultwprime, pmax ? pmax : defaultpmax);
+                    ZoneRange range(begin, end, cp, ftp ? ftp : cp, wprime ? wprime : defaultwprime, pmax ? pmax : defaultpmax);
                     range.zones = zoneInfos;
 
                     if (range.zones.empty()) {
@@ -240,6 +242,21 @@ bool Zones::read(QFile &file)
             }
         }
 
+        // check for FTP
+        if (ftpx.indexIn(line, 0) != -1) {
+            if (!in_range)
+                qDebug()<<"ignoring errant FTP= in power.zones";
+            else {
+                ftp = ftpx.cap(1).toInt();
+
+                // ok its stored, so if it is in kJ upscale
+                // if it is zero as never set, then use default
+                if (!ftp) {
+                    ftp = cp;
+                }
+            }
+        }
+
         // check for w'
         if (wprimerx.indexIn(line, 0) != -1) {
             if (!in_range)
@@ -355,7 +372,7 @@ next_line: {}
 
     if (in_range) {
 
-        ZoneRange range(begin, end, cp, wprime ? wprime : defaultwprime, pmax ? pmax : defaultpmax);
+        ZoneRange range(begin, end, cp, ftp ? ftp : cp, wprime ? wprime : defaultwprime, pmax ? pmax : defaultpmax);
         range.zones = zoneInfos;
 
         if (range.zones.empty()) {
@@ -548,6 +565,12 @@ int Zones::getCP(int rnum) const
     return ranges[rnum].cp;
 }
 
+int Zones::getFTP(int rnum) const
+{
+    assert(rnum < ranges.size());
+    return ranges[rnum].ftp;
+}
+
 int Zones::getWprime(int rnum) const
 {
     assert(rnum < ranges.size());
@@ -566,6 +589,12 @@ void Zones::setCP(int rnum, int cp)
     modificationTime = QDateTime::currentDateTime();
 }
 
+void Zones::setFTP(int rnum, int ftp)
+{
+    ranges[rnum].ftp = ftp;
+    modificationTime = QDateTime::currentDateTime();
+}
+
 void Zones::setWprime(int rnum, int wprime)
 {
     ranges[rnum].wprime = wprime;
@@ -759,6 +788,7 @@ void Zones::write(QDir home)
     for (int i = 0; i < ranges.size(); i++) {
 
         int cp = getCP(i);
+        int ftp = getFTP(i);
         int wprime = getWprime(i);
         int pmax = getPmax(i);
 
@@ -772,6 +802,8 @@ void Zones::write(QDir home)
         strzones += QString("%1: CP=%2").arg(getStartDate(i).toString("yyyy/MM/dd")).arg(cp);
         strzones += QString("\n");
 
+        // wite out the FTP value
+        strzones += QString("FTP=%1\n").arg(ftp);
         // wite out the W' value
         strzones += QString("W'=%1\n").arg(wprime);
         // wite out the Pmax value
@@ -834,14 +866,14 @@ void Zones::write(QDir home)
     }
 }
 
-void Zones::addZoneRange(QDate _start, QDate _end, int _cp, int _wprime, int _pmax)
+void Zones::addZoneRange(QDate _start, QDate _end, int _cp, int _ftp, int _wprime, int _pmax)
 {
-    ranges.append(ZoneRange(_start, _end, _cp, _wprime, _pmax));
+    ranges.append(ZoneRange(_start, _end, _cp, _ftp, _wprime, _pmax));
 }
 
 // insert a new zone range using the current scheme
 // return the range number
-int Zones::addZoneRange(QDate _start, int _cp, int _wprime, int _pmax)
+int Zones::addZoneRange(QDate _start, int _cp, int _ftp, int _wprime, int _pmax)
 {
     int rnum;
 
@@ -849,8 +881,8 @@ int Zones::addZoneRange(QDate _start, int _cp, int _wprime, int _pmax)
     for(rnum=0; rnum < ranges.count(); rnum++) if (ranges[rnum].begin > _start) break;
 
     // at the end ?
-    if (rnum == ranges.count()) ranges.append(ZoneRange(_start, date_infinity, _cp, _wprime, _pmax));
-    else ranges.insert(rnum, ZoneRange(_start, ranges[rnum].begin, _cp, _wprime, _pmax));
+    if (rnum == ranges.count()) ranges.append(ZoneRange(_start, date_infinity, _cp, _ftp, _wprime, _pmax));
+    else ranges.insert(rnum, ZoneRange(_start, ranges[rnum].begin, _cp, _ftp, _wprime, _pmax));
 
     // modify previous end date
     if (rnum) ranges[rnum-1].end = _start;
@@ -961,6 +993,9 @@ Zones::getFingerprint() const
         // CP
         x += ranges[i].cp;
 
+        // FTP
+        x += ranges[i].ftp;
+
         // W'
         x += ranges[i].wprime;
 
@@ -991,6 +1026,9 @@ Zones::getFingerprint(QDate forDate) const
         // CP
         x += ranges[i].cp;
 
+        // FTP
+        x += ranges[i].ftp;
+
         // W'
         x += ranges[i].wprime;
 
