@@ -59,14 +59,15 @@ struct ZoneInfo {
 struct ZoneRange {
     QDate begin, end;
     int cp;
+    int ftp;
     int wprime; // aka awc
     int pmax;
     QList<ZoneInfo> zones;
     bool zonesSetFromCP;
     ZoneRange(const QDate &b, const QDate &e) :
-        begin(b), end(e), cp(0), wprime(0), pmax(0), zonesSetFromCP(false) {}
-    ZoneRange(const QDate &b, const QDate &e, int _cp, int _wprime, int pmax) :
-        begin(b), end(e), cp(_cp), wprime(_wprime), pmax(pmax), zonesSetFromCP(false) {}
+        begin(b), end(e), cp(0), ftp(0), wprime(0), pmax(0), zonesSetFromCP(false) {}
+    ZoneRange(const QDate &b, const QDate &e, int _cp, int _ftp, int _wprime, int pmax) :
+        begin(b), end(e), cp(_cp), ftp(_ftp), wprime(_wprime), pmax(pmax), zonesSetFromCP(false) {}
 
     // used by qSort()
     bool operator< (ZoneRange right) const {
@@ -124,8 +125,8 @@ class Zones : public QObject
         int getRangeSize() const;
 
         // Add ranges
-        void addZoneRange(QDate _start, QDate _end, int _cp, int _wprime, int _pmax);
-        int addZoneRange(QDate _start, int _cp, int _wprime, int _pmax);
+        void addZoneRange(QDate _start, QDate _end, int _cp, int _ftp, int _wprime, int _pmax);
+        int addZoneRange(QDate _start, int _cp, int _ftp, int _wprime, int _pmax);
         void addZoneRange();
 
         // Get / Set ZoneRange details
@@ -135,6 +136,8 @@ class Zones : public QObject
         // get and set CP for a given range
         int getCP(int rnum) const;
         void setCP(int rnum, int cp);
+        int getFTP(int rnum) const;
+        void setFTP(int rnum, int ftp);
         int getWprime(int rnum) const;
         void setWprime(int rnum, int wprime);
         int getPmax(int rnum) const;
