@@ -16,18 +16,26 @@
  * Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+#include "Context.h"
+#include "Athlete.h"
+#include "MainWindow.h"
+#include "Settings.h"
+#include "TrainDB.h"
+#include "Colors.h"
+#include "GcUpgrade.h"
+
 #include <QApplication>
 #include <QtGui>
 #include <QFile>
 #include <QWebSettings>
 #include <QMessageBox>
 #include "ChooseCyclistDialog.h"
-#include "MainWindow.h"
-#include "Settings.h"
-#include "TrainDB.h"
-#include "Colors.h"
+#ifdef GC_WANT_HTTP
+#include "httplistener.h"
+#include "httprequesthandler.h"
+#endif
 
-#include "GcUpgrade.h"
+#include <signal.h>
 
 // redirect errors to `home'/goldencheetah.log
 // sadly, no equivalent on Windows
@@ -64,6 +72,44 @@ bool restarting = false;
 // root directory shared by all
 QString gcroot;
 
+QApplication *application;
+
+bool nogui;
+#ifdef GC_WANT_HTTP
+#include "APIWebService.h"
+#if QT_VERSION > 0x50000
+void myMessageOutput(QtMsgType type, const QMessageLogContext &, const QString &string)
+ {
+    const char *msg = string.toLocal8Bit().constData();
+#else
+void myMessageOutput(QtMsgType type, const char *msg)
+ {
+#endif
+     //in this function, you can write the message to any stream!
+     switch (type) {
+     case QtDebugMsg:
+         fprintf(stderr, "Debug: %s\n", msg);
+         break;
+     case QtWarningMsg: // supress warnings unless server mode
+         if (nogui) fprintf(stderr, "Warning: %s\n", msg);
+         break;
+     case QtCriticalMsg:
+         fprintf(stderr, "Critical: %s\n", msg);
+         break;
+     case QtFatalMsg:
+         fprintf(stderr, "Fatal: %s\n", msg);
+         abort();
+     }
+ }
+
+HttpListener *listener;
+void sigabort(int)
+{
+    qDebug()<<""; // newline
+    application->exit();
+}
+#endif
+
 int
 main(int argc, char *argv[])
 {
@@ -83,7 +129,8 @@ main(int argc, char *argv[])
 #else
     bool debug = false;
 #endif
-
+    bool server = false;
+    nogui = false;
     bool help = false;
 
     // honour command line switches
@@ -95,6 +142,9 @@ main(int argc, char *argv[])
             help = true;
             fprintf(stderr, "GoldenCheetah %s (%d)\nusage: GoldenCheetah [[directory] athlete]\n\n", VERSION_STRING, VERSION_LATEST);
             fprintf(stderr, "--help or --version to print this message and exit\n");
+#ifdef GC_WANT_HTTP
+            fprintf(stderr, "--server to run as an API server\n");
+#endif
 #ifdef GC_DEBUG
             fprintf(stderr, "--debug             to turn on redirection of messages to goldencheetah.log [debug build]\n");
 #else
@@ -103,6 +153,14 @@ main(int argc, char *argv[])
             fprintf (stderr, "\nSpecify the folder and/or athlete to open on startup\n");
             fprintf(stderr, "If no parameters are passed it will reopen the last athlete.\n\n");
 
+        } else if (arg == "--server") {
+#ifdef GC_WANT_HTTP
+                nogui = server = true;
+#else
+                fprintf(stderr, "HTTP support not compiled in, exiting.\n");
+                exit(1);
+#endif
+
         } else if (arg == "--debug") {
 
 #ifdef GC_DEBUG
@@ -127,6 +185,9 @@ main(int argc, char *argv[])
     //
     // INITIALISE ONE TIME OBJECTS
     //
+#ifdef GC_WANT_HTTP
+    listener = NULL;
+#endif
 
 #ifdef Q_OS_X11
     XInitThreads();
@@ -142,22 +203,24 @@ main(int argc, char *argv[])
 #endif
 
     // create the application -- only ever ONE regardless of restarts
-    QApplication *application = new QApplication(argc, argv);
+    application = new QApplication(argc, argv);
 
 #ifdef Q_OS_MAC
     // get an autorelease pool setup
     static CocoaInitializer cocoaInitializer;
 #endif
 
+    // set default colors
+    GCColor::setupColors();
+    appsettings->migrateQSettingsSystem(); // colors must be setup before migration can take place, but reading has to be from the migrated ones
+    GCColor::readConfig();
+
     // set defaultfont
     QFont font;
     font.fromString(appsettings->value(NULL, GC_FONT_DEFAULT, QFont().toString()).toString());
     font.setPointSize(appsettings->value(NULL, GC_FONT_DEFAULT_SIZE, 10).toInt());
     application->setFont(font); // set default font
 
-    // set default colors
-    GCColor::setupColors();
-    GCColor::readConfig();
 
     //
     // OPEN FIRST MAINWINDOW
@@ -223,10 +286,14 @@ main(int argc, char *argv[])
 
         // set global root directory
         gcroot = home.canonicalPath();
+        appsettings->initializeQSettingsGlobal(gcroot);
+
 
         // now redirect stderr
 #ifndef WIN32
         if (!debug) nostderr(home.canonicalPath());
+#else
+        Q_UNUSED(debug)
 #endif
 
         // install QT Translator to enable QT Dialogs translation
@@ -254,10 +321,11 @@ main(int argc, char *argv[])
 
         // lets do what the command line says ...
         QVariant lastOpened;
-        if(args.count() == 2) { // $ ./GoldenCheetah Mark
+        if(args.count() == 2) { // $ ./GoldenCheetah Mark -or- ./GoldenCheetah --server ~/athletedir
 
             // athlete
-            lastOpened = args.at(1);
+            if (!server) lastOpened = args.at(1);
+            else home.cd(args.at(1));
 
         } else if (args.count() == 3) { // $ ./GoldenCheetah ~/Athletes Mark
 
@@ -274,12 +342,89 @@ main(int argc, char *argv[])
             // no parameters passed lets open the last athlete we worked with
             lastOpened = appsettings->value(NULL, GC_SETTINGS_LAST);
 
-            // but hang on, did they crash? if so we need to open with a menu
-            if(appsettings->cvalue(lastOpened.toString(), GC_SAFEEXIT, true).toBool() != true)
+            // does lastopened Directory exists at all
+            QDir lastOpenedDir(gcroot+"/"+lastOpened.toString());
+            if (lastOpenedDir.exists()) {
+                // but hang on, did they crash? if so we need to open with a menu
+                appsettings->initializeQSettingsAthlete(gcroot, lastOpened.toString());
+                if(appsettings->cvalue(lastOpened.toString(), GC_SAFEEXIT, true).toBool() != true)
+                    lastOpened = QVariant();
+            } else {
                 lastOpened = QVariant();
+            }
             
         }
 
+#ifdef GC_WANT_HTTP
+
+        // The API server offers webservices (default port 12021, see httpserver.ini)
+        // This is to enable integration with R and similar
+        if (appsettings->value(NULL, GC_START_HTTP, true).toBool() || server) {
+
+            // notifications etc
+            if (nogui) {
+                qDebug()<<"Starting GoldenCheetah API web-services... (hit ^C to close)";
+                qDebug()<<"Athlete directory:"<<home.absolutePath();
+            } else {
+                // switch off warnings if in gui mode
+#ifndef GC_WANT_ALLDEBUG
+#if QT_VERSION > 0x50000
+                qInstallMessageHandler(myMessageOutput);
+#else
+                qInstallMsgHandler(myMessageOutput);
+#endif
+#endif
+            }
+
+            QString httpini = home.absolutePath() + "/httpserver.ini";
+            if (!QFile(httpini).exists()) {
+
+                // read default ini file
+                QFile file(":webservice/httpserver.ini");
+                QString content;
+                if (file.open(QIODevice::ReadOnly)) {
+                    content = file.readAll();
+                    file.close();
+                }
+
+                // write default ini file
+                QFile out(httpini);
+                if (out.open(QIODevice::WriteOnly)) {
+                
+                    out.resize(0);
+                    QTextStream stream(&out);
+                    stream << content;
+                    out.close();
+                }
+            }
+
+            // use the default handler (just get an error page)
+            QSettings* settings=new QSettings(httpini,QSettings::IniFormat,application);
+
+            if (listener) {
+                // when changing the Athlete Directory, there is already a listener running
+                // close first to avoid errors
+                listener->close();
+            }
+            listener=new HttpListener(settings,new APIWebService(home, application),application);
+
+            // if not going on to launch a gui...
+            if (nogui) {
+                // catch ^C exit
+                signal(SIGINT, sigabort);
+
+                ret = application->exec();
+
+                // stop web server if running
+                qDebug()<<"Stopping GoldenCheetah API web-services...";
+                listener->close();
+
+                // and done
+                exit(0);
+            }
+        }
+#endif
+
         // lets attempt to open as asked/remembered
         bool anyOpened = false;
         if (lastOpened != QVariant()) {
@@ -287,7 +432,9 @@ main(int argc, char *argv[])
             QStringListIterator i(list);
             while (i.hasNext()) {
                 QString cyclist = i.next();
+                QString homeDir = home.canonicalPath();
                 if (home.cd(cyclist)) {
+                    appsettings->initializeQSettingsAthlete(homeDir, cyclist);
                     GcUpgrade v3;
                     if (v3.upgradeConfirmedByUser(home)) {
                         MainWindow *mainWindow = new MainWindow(home);
@@ -317,12 +464,14 @@ main(int argc, char *argv[])
             }
 
             // chosen, so lets get the choice..
+            QString homeDir = home.canonicalPath();
             home.cd(d.choice());
             if (!home.exists()) {
                 delete trainDB;
                 exit(0);
             }
 
+            appsettings->initializeQSettingsAthlete(homeDir, d.choice());
             // .. and open a mainwindow
             GcUpgrade v3;
             if (v3.upgradeConfirmedByUser(home)) {
@@ -340,6 +489,9 @@ main(int argc, char *argv[])
         // close trainDB
         delete trainDB;
 
+        // reset QSettings (global & Athlete)
+        appsettings->clearGlobalAndAthletes();
+
         // clear web caches (stop warning of WebKit leaks)
         QWebSettings::clearMemoryCaches();
 
